ColWidth	equ	20				; Width of collision map in bytes

;************************************************************************************************************************
;
;   Name    :  InitScreen
;   Function:  Init the screen rendering mode
;
;	In:		A = Level number
;
;************************************************************************************************************************
InitScreen:
		sta	LevelNumber

		jsr	DrawScreen
		rts




;************************************************************************************************************************
;
;   Name    :  ClearScreen
;   Function:  Wipe the screen completely
;
;************************************************************************************************************************
ClearScreen:
		lda	#0
		sta	FillPattern
		lda	#$00
		sta	Colour1
		lda	#$00
		sta	Colour2
		jmp	Cls




;************************************************************************************************************************
;
;   Name    :  DrawScreen
;   Function:  Draw the desired level
;
;************************************************************************************************************************
BitMap		equ	Temp
Screen		equ	Temp+2
Bitmap2		equ	Temp+4
BlockCount	equ	Temp+6
Block		equ	Temp+7
BlockCalc	equ	Temp+9
ColourScr1	equ	Temp+10
ColourScr2	equ	Temp+12
YCount		equ	Temp+14
BombX		equ	Temp+15
BombY		equ	Temp+16
BombPos		equ	Temp+17
BombIndex	equ	Temp+19

DrawScreen:
		lda	#Lo(BitmapScreen)				; point to bitmap screen
		sta	Bitmap
		lda	#Hi(BitmapScreen)
		sta	Bitmap+1

		lda	#Lo(ScreenData1)				; point to level data (for runtime collision detection)
		sta	Map
		lda	#Hi(ScreenData1)
		sta	Map+1

		lda	#Lo(ScreenData1+ColWidth)		; point to 1 line down for drawing level
		sta	Screen							; as top+bottom line are
		lda	#Hi(ScreenData1+ColWidth)
		sta	Screen+1
		
		lda	#Lo(HWScreen1)					; point to colour screen 1
		sta	ColourScr1
		lda	#Hi(HWScreen1)
		sta	ColourScr1+1

		lda	#Lo($d800)						; point to HARDWARE colour screen
		sta	ColourScr2
		lda	#Hi($d800)
		sta	ColourScr2+1

		lda	#12
		sta	YCount							; Number of rows to render
		lda	#0
		sta	BombX							; Bomb X and Y coords
		sta	BombY
		sta	BombIndex						; bomb number
		

		ldy	#0
DrawScreenLoop
		sty	BlockCount
		lda	#0
		sta	BlockCalc
		lda	(Screen),y			; Get tile
		bne	!GotTile
		jmp	!DoNextTile
!GotTile:
		cmp	#$ff
		bne	!NormalTile
		jsr	AddBomb
		jmp	!DoNextTile

!NormalTile:
		asl
		rol	BlockCalc
		asl
		rol	BlockCalc
		asl
		rol	BlockCalc
		sta	Block				; Store *8 in Block
		lda	BlockCalc
		sta	Block+1

		lda	Block				; Now *4 more to make *32
		asl
		rol	BlockCalc
		asl
		rol	BlockCalc

		clc						; Now add *8 + *32 together for *40
		adc	Block				; We now have the block address
		sta	Block			
		lda	BlockCalc
		adc	Block+1
		sta	Block+1

		clc						; Now add on base address of tile
		lda	#Lo(TileData)		; and we now have the tile to draw
		adc	Block
		sta	Block
		lda	#Hi(TileData)
		adc	Block+1
		sta	Block+1


		ldy	#15					; Copy 1st row (16x8 pixels)
!CopyTopOfTile:
		lda	(Block),y
		sta	(Bitmap),y
		dey
		bpl	!CopyTopOfTile

		; Move down a character line on the bitmap screen
		clc
		lda	Bitmap
		adc	#Lo(320)
		sta	Bitmap2
		lda	Bitmap+1
		adc	#Hi(320)
		sta	Bitmap2+1

		; Move onto second row...
		clc
		lda	Block
		adc	#16
		sta	Block
		bcc	!SkipCarry
		inc	Block+1
!SkipCarry

		ldy	#15					; Copy 2nd row of 16x8 pixels
!CopyTopOfTile2:
		lda	(Block),y
		sta	(Bitmap2),y
		dey
		bpl	!CopyTopOfTile2

		; Now move onto COLOURS
		clc
		lda	Block
		adc	#16
		sta	Block
		bcc	!SkipCarry2
		inc	Block+1
!SkipCarry2
		ldy	#0
		lda	(Block),y
		sta	(ColourScr1),y
		iny
		lda	(Block),y
		sta	(ColourScr1),y
		iny
		lda	(Block),y
		pha
		iny
		lda	(Block),y
		ldy	#41
		sta	(ColourScr1),y
		pla
		dey
		sta	(ColourScr1),y

		; Now move onto Colour RAM colours...
		clc
		lda	Block
		adc	#4
		sta	Block
		bcc	!SkipCarry3
		inc	Block+1
!SkipCarry3

		; Copy colour RAM colours
		ldy	#0
		lda	(Block),y
		sta	(ColourScr2),y
		iny
		lda	(Block),y
		sta	(ColourScr2),y
		iny
		lda	(Block),y
		pha
		iny
		lda	(Block),y
		ldy	#41
		sta	(ColourScr2),y
		pla
		dey
		sta	(ColourScr2),y
!DoNextTile:
		clc
		lda	Bitmap				; Move bitmap location on
		adc	#Lo(16)
		sta	Bitmap
		lda	Bitmap+1
		adc	#Hi(16)
		sta	Bitmap+1

		clc
		lda	ColourScr1			; Move colour screen 1 on
		adc	#Lo(2)
		sta	ColourScr1
		bcc	!SkipInc4
		inc	ColourScr1+1
!SkipInc4:

		clc
		lda	ColourScr2			; Move colour screen 2 on
		adc	#Lo(2)
		sta	ColourScr2
		bcc	!SkipInc5
		inc	ColourScr2+1
!SkipInc5:

		inc	BombX
		ldy	BlockCount
		iny
		cpy	#ColWidth
		jne	DrawScreenLoop


		lda	#0
		sta	BombX
		inc	BombY

		clc
		lda	Bitmap				; Move bitmap location on
		adc	#Lo(320)
		sta	Bitmap
		lda	Bitmap+1
		adc	#Hi(320)
		sta	Bitmap+1

		clc
		lda	ColourScr1			; Move colour screen 1 on
		adc	#40
		sta	ColourScr1
		bcc	!SkipInc6
		inc	ColourScr1+1
!SkipInc6:

		clc
		lda	ColourScr2			; Move colour screen 1 on
		adc	#40
		sta	ColourScr2
		bcc	!SkipInc7
		inc	ColourScr2+1
!SkipInc7:


		ldy	#0
		

		clc						; move to next row in level map
		lda	Screen
		adc	#ColWidth
		sta	Screen
		lda	Screen+1
		adc	#0
		STa	Screen+1

		dec	YCount
		jne	DrawScreenLoop

		rts

;************************************************************************************************************************
;
;   Name    :	AddBomb
;   Function:   Add a sprite bomb to the level. This takes the current X,Y bomb coord and makes real
;				sprite coordinates from it.
;
;************************************************************************************************************************
AddBomb:
		ldx	BombIndex
		lda	#0				; Get "X"
		sta	BombPos+1
		lda	BombX
		asl
		rol	BombPos+1
		asl
		rol	BombPos+1
		asl
		rol	BombPos+1
		asl
		rol	BombPos+1	
		clc
		adc	#24
		sta	SpXLo+1,x
		lda	#0
		adc	BombPos+1
		sta	SpXHi+1,x

		lda	BombY
		asl
		asl
		asl
		asl
		clc
		adc	#50
		sta	SpY+1,x

		lda	#EmptySpriteShape+1
		sta	SpShape+1,x
		lda	#2
		sta	SpCol+1,x

		lda	#8/2
		sta	CenterXoffset+1,x
		lda	#8
		sta	CenterYoffset+1,x
		lda	#6/2
		sta	RadX+1,x
		lda	#6
		sta	RadY+1,x

		inx
		stx	BombIndex
		rts

		
		
		
; Collision map "Y" multiply table
MapXplyTable_Lo:
		db	Lo(0*20)
		db	Lo(1*20)
		db	Lo(2*20)
		db	Lo(3*20)
		db	Lo(4*20)
		db	Lo(5*20)
		db	Lo(6*20)
		db	Lo(7*20)
		db	Lo(8*20)
		db	Lo(9*20)
		db	Lo(10*20)
		db	Lo(11*20)
		db	Lo(12*20)
		db	Lo(13*20)
MapXplyTable_Hi:
		db	Hi(0*20)
		db	Hi(1*20)
		db	Hi(2*20)
		db	Hi(3*20)
		db	Hi(4*20)
		db	Hi(5*20)
		db	Hi(6*20)
		db	Hi(7*20)
		db	Hi(8*20)
		db	Hi(9*20)
		db	Hi(10*20)
		db	Hi(11*20)
		db	Hi(12*20)
		db	Hi(13*20)


;
; Level tile map. We have a 1 tile border which isn't draw
; to allow for simple collision
;
; 0 = space
; >0 = tile
; $ff = bomb
;
ScreenData1:
		db	$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01
		db	$ff,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		db	$00,$00,$00,$01,$01,$01,$01,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$ff
		db	$ff,$00,$00,$00,$ff,$00,$ff,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		db	$00,$00,$00,$00,$00,$00,$00,$00,$ff,$00,$ff,$00,$ff,$00,$00,$00,$00,$00,$00,$ff
		db	$ff,$00,$00,$00,$00,$00,$00,$01,$01,$01,$01,$01,$01,$01,$00,$00,$00,$00,$00,$00
		db	$00,$00,$00,$00,$00,$00,$00,$00,$ff,$00,$ff,$00,$ff,$00,$00,$00,$00,$00,$00,$ff
		db	$ff,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		db	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$01,$01,$01,$01,$01,$00,$ff
		db	$ff,$00,$00,$ff,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		db	$00,$00,$01,$01,$01,$01,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$ff
		db	$ff,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		db	$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
		db	$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01





TileData
		db	0,0,0,0,0,0,0,0		; Blocks are 16x16
		db	0,0,0,0,0,0,0,0		; Blocks are stored
		db	0,0,0,0,0,0,0,0		; AB
		db	0,0,0,0,0,0,0,0		; CD
		db	$11,$11,$11,$11		; Colour screen (2 cols)
		db	$01,$01,$01,$01		; colour screen2 (1 col)

		;
		; Default "round" platform
		;
		db	%10101010
		db	%10101010
		db	%10011001
		db	%01100110
		db	%01010101
		db	%11011101
		db	%01110111
		db	%11111111

		db	%10101010
		db	%10101010
		db	%10011001
		db	%01100110
		db	%01010101
		db	%11011101
		db	%01110111
		db	%11111111

		db	%11111111
		db	%01110111
		db	%11011101
		db	%01010101
		db	%01100110
		db	%10011001
		db	%10101010
		db	%10101010

		db	%11111111
		db	%01110111
		db	%11011101
		db	%01010101
		db	%01100110
		db	%10011001
		db	%10101010
		db	%10101010

		db	$71,$71			; Colours 1
		db	$29,$29

		db	$08,$08			; Colours 2
		db	$08,$08		

