;****************************************************************************************
;
;                                 Main Loop
;
;****************************************************************************************
MainLoopInit:
			jsr  Init				; Initialise the game



MainLoop	lda	#$00				; wait VBlank
			sta	VBlank				; IRQ will change this from a $00 
WaitVblank	lda	VBlank				; to $ff so we just need to wait...
			beq	WaitVblank

!DoPause
			jsr	ScanKeys            ; scan joystick/keys...


			lda	GameKeys			; SHIFT or SHIFT-LOCK will pause
			and	#KEY_LEFT_SHIFT
			bne	!DoPause

			;
			; Your code here....
			;
			jsr	ProcessKeys
			jsr	MovePlayer
			jsr	DoSpriteCollision


WaitFrames:
			lda	FrameCounter			; Wait for the desired number of frames to pass
			cmp	#FramesPerGameCycle		; 0=50FPS, 1=25fps
			bcc	WaitFrames
			jmp	MainLoop





;************************************************************************************************************************
;
;   Name    :  Init
;   Function:  Init the main loop
;
;************************************************************************************************************************
Init:
			;
			; Clear all sprites
			;
			ldx	#MaxSprites-1
			lda	#0
!ClearAll:	sta	SpY,x
			dex
			bpl	!ClearAll
!FinishedSetup:
			;
			; Setup player...
			;
			lda	#100+24
			sta	SpXLo
			lda	#50+100-10
			sta	SpY
			lda	#0
			sta	SpXHi
			lda	#EmptySpriteShape+2
			sta	SpShape
			
			lda	#1
			sta	$D025
			lda	#0
			sta	$D026
			jsr	InitPanel
			jsr	InitScreen
			rts

;*****************************************************************************
;
; Init the panel: Black bar at the bottom of the screen
;
;*****************************************************************************
InitPanel:
			lda	#Lo(BitmapScreen+6000-320)
			sta	Temp
			lda	#Hi(BitmapScreen+6000-320)
			sta	Temp+1

			ldx	#0
!Lp1
			lda	#$ff
			sta	BitmapScreen+8000-320,x
			sta	BitmapScreen+8000-256,x
			dex
			bne	!Lp1

			ldx	#39
!Lp2
			lda	#$00
			sta	$d800+(40*24),x
			sta	HWScreen1+(40*24),x
			dex
			bpl	!Lp2
			rts


;*****************************************************************************
;
; Move player around the screen
;
;*****************************************************************************
SpeedX		EQU	4
MaxYSpeed	equ	10
FirePressed	db	0
MovePlayer:
;			lda	Joystick
;			and	#JOY_UP
;			beq	!TryDown

;!TryDown:	lda	Joystick
;			and	#JOY_DOWN
;			beq	!TryLeft

!TryLeft:	lda	Joystick
			and	#JOY_LEFT
			beq	!TryRight
			lda	#Lo(-SpeedX)
			sta	XSpeed
			lda	#Hi(-SpeedX)
			sta	XSpeed+1


!TryRight:	lda	Joystick
			and	#JOY_RIGHT
			beq	!NoMovement
			lda	#Lo(SpeedX)
			sta	XSpeed
			lda	#Hi(SpeedX)
			sta	XSpeed+1


!NoMovement
			lda	Joystick
			and	#JOY_FIRE
			beq	NoFire
dofire
			lda	FirePressed
			bne	NoFire2
			lda	YSpeed
			bne	NoFire
			lda	#-(MaxYSpeed*4)
			sta	YSpeed
NoFire:		sta	FirePressed
NoFire2:

			inc	SpCol
			;jmp	DoPlayerMovement	;If not a fall through, we need this


;*****************************************************************************
; Function:	Process Game Keys
;*****************************************************************************
ColX		equ	Temp
ColY		equ	Temp+1
ColMap		equ	Temp+2
YFrac		equ	Temp+4
XFrac		equ	Temp+5
									
DoPlayerMovement:
			;
			; First we move the player by adding on XSpeed and YSpeed
			;
			lda	#0
			sta	Temp
			lda	Yspeed			; /Yspeed by 2 (lose lower bit)
			bpl	!okay
			dec	Temp
!okay		lsr	Temp
			ror
			lsr	Temp
			ror
			clc					; Apply current speed on Y
			adc	SpY
			sta	SpY

			clc					; Apply current speed on X
			lda	SpXLo
			adc	Xspeed
			sta	SpXLo
			lda	SpXHi  
			adc	XSpeed+1
			and	#1
			sta	SpXHi  

			; Check player collision to screen edge
			lda	XSpeed+1
			bpl	!TestRight
			lda	SpXHi
			bne	!NoReset
			lda	SpXLo
			cmp	#24
			bcs	!NoReset
			lda	#24
			sta	SpXLo
			jmp	!NoReset
!TestRight
			lda	SpXHi
			beq	!NoReset
			lda	SpXLo
			cmp	#Lo(320-16+24)
			bcc	!NoReset
			lda	#Lo(320-16+24)
			sta	SpXLo
!NoReset


			;
			; Apply gravity to YSpeed
			;
			clc	
			lda	Yspeed
			adc	#2
			cmp	#MaxYSpeed*4
			bmi	!StoreY
			bcc	!StoreY
			lda	#MaxYSpeed*4
!StoreY		sta	Yspeed



			;
			; Next transform sprite coordinate into a collision map address
			;	
			lda	SpXHi		; First get the
			lsr
			lda	SpXLo
			ror
			lsr
			lsr
			sec
			sbc	#3			; 8 pixels off LEFT edge (border)
			lsr				; /16 so we can do collision to 16x16's
			sta	ColX

			sec
			lda	SpY			
			sbc	#50-16
			sta	YFrac
			lsr
			lsr
			lsr
			lsr				; /16 for collision map
			tay
			sta	ColY


			clc
			lda	MapXplyTable_lo,y
			adc	ColX
			sta	ColMap
			lda	MapXplyTable_Hi,y
			adc	#0
			sta	ColMap+1

			clc
			lda	Map
			adc	ColMap
			sta	ColMap
			lda	Map+1
			adc	ColMap+1
			sta	ColMap+1

			
			;
			; Now work out the X and Y fraction into a block
			;
			sec
			lda	SpXLo
			sbc	#8
			and	#$0f
			sta	XFrac
				
			lda	YFrac
			and	#$0f
			sta	YFrac


			;
			; Now do collision!
			;
			lda	YSpeed				; First are we falling? If YSpeed >=0 then yes
			bpl	TestFalling
			ldy	#0					; if YSpeed <0 then we're going up
			lda	(ColMap),y			; so check above (well check the block we're IN)
			beq	!NextCheck			; if ==0 then NO collision
			bpl	NotEmptyMoveDown	; if >0 then force DOWN onto block boundary
!NextCheck
			lda	XFrac				; if we're half way between a block,
			beq	NextColl			; we need to check the next block along as well
			iny
			lda	(ColMap),y			; Same rules as above apply
			beq	NextColl
			bmi	NextColl
NotEmptyMoveDown
			sec						; If we've hit something then we need to 
			lda	#16					; force the player to be UNDER the block
			sbc	YFrac
			clc
			adc	SpY
			sta	SpY
			lda	#0					; need to kill the YSpeed and YFrac
			sta	YFrac
			sta	YSpeed
			clc						; We also need to move the collision map pointer to
			lda	ColMap				; point to the new row, or the X collision will go funny.
			adc	#22
			sta	ColMap
			bcc	!skiper
			inc	ColMap+1
!skiper
			jmp	TestX

TestFalling:
			ldy	#ColWidth			; Same as Going up, but checking under the player
			lda	(ColMap),y
			beq	!CheckNext
			bpl	NotEmptyMoveUp
!CheckNext:
			lda	XFrac
			beq	NextColl
			iny
			lda	(ColMap),y
			beq	NextColl
			bmi	NextColl
NotEmptyMoveUp
			sec						; If theres a block under us then force 
			lda	SpY					; the player to sit ON the block
			sec
			sbc	YFrac
			sta	SpY
			lda	#0					; need to kill the YSpeed and YFrac
			sta	YFrac
			sta	Yspeed
NextColl

			; Now we do exactly the same for "X" as we did "Y"
TestX		
			lda	XSpeed+1			; if XSpeed+1 != 0 then its negative, which means we're moving LEFT
			bne	TestLeft
			lda	XSpeed				; if XSpeed==0 then we're not moving on X so dont check
			beq	NoXTest

			; Test RIGHT
			ldy	#1					; otherwise we're going RIGHT
			lda	(ColMap),y			; Check the block to the RIGHT
			beq	!CheckNext
			bpl	NotEmptyMoveLeft
!CheckNext:
			lda	YFrac				; if YFrac !=0 then check the block below too
			beq	NoXTest
			ldy	#ColWidth+1
			lda	(ColMap),y
			beq	NoXTest
			bmi	NoXTest
NotEmptyMoveLeft
			sec						; if we hit something, then we need to force the player
			lda	SpXLo				; BACK so hes before this block
			sbc	XFrac
			sta	SpXLo
			lda	SpXHI
			sbc	#0
			and	#1
			sta	SpXHi
			lda	#0
			sta XFrac
			jmp	NoXTest


			; Test LEFT
TestLeft:
			ldy	#0					; Now do the same for LEFT
			lda	(ColMap),y
			beq	!NextCheck
			bpl	NotEmptyMoveRight
!NextCheck:
			lda	YFrac
			beq	NoXTest
			ldy	#ColWidth
			lda	(ColMap),y
			beq	NoXTest
			bmi	NoXTest
NotEmptyMoveRight
			sec						; If we hit something we need to force the player to
			lda	#16					; be on the RIGHT of this block.
			sbc	XFrac
			clc
			adc	SpXLo
			sta	SpXLo
			lda	#0
			adc	SpXHi
			and	#1
			sta	SpXHi
			lda	#0
			sta XFrac
			inc	ColMap
			bne	!skip
			inc	ColMap
!skip		

NoXTest
ExitCollision
			lda	#0					; XSpeed is reset every frame.
			sta	xSpeed
			sta	xSpeed+1


			rts




;*****************************************************************************
; Function:	Process Game Keys
;*****************************************************************************
ProcessKeys:
			lda	GameKeys
			bne	!KeysPressed
			rts
!KeysPressed:
			and	#KEY_F1
			beq	!F1_NotPressed
			lda	#1
			sta	$d021
	
!F1_NotPressed:
			lda	GameKeys
			and	#KEY_F3
			beq	!F3_NotPressed
			lda	#0
			sta	$d021
!F3_NotPressed:
			rts



;*****************************************************************************
; XPLY routine
;
; enter : xplyc = xp 1 ; xplyd = xp 2
;
;		  XPLYC can NOT be zero!!
;
; exit  : A = ans hi / xplyc = ans lo 
;
; used  : a
;
; Totals including RTS (not including jsr to here)
; 119 worst
; 101 best
;*****************************************************************************
XPLY:		
		LDA #$00		; 2
		dec	xplyc		; 5  pre-decrement to offset a SET carry when adding "xplyc"

		lsr xplyd		; 5
		BCC !skip1		; 2
		ADC xplyc		; 3	=10
!skip1
		ROR				; 2
		ROR xplyd		; 5
		BCC !skip2		; 3 = 10 (*9=90 best)
		ADC xplyc		;
!skip2
		ROR				; 2
		ROR xplyd		; 5
		BCC !skip3		; 2
		ADC xplyc		; 3 = 12 (*8=96 worst)
!skip3
		ROR
		ROR xplyd
		BCC !skip4
		ADC xplyc
!skip4
		ROR
		ROR xplyd
		BCC !skip5
		ADC xplyc
!skip5
		ROR
		ROR xplyd
		BCC !skip6
		ADC xplyc
!skip6
		ROR
		ROR xplyd
		BCC !skip7
		ADC xplyc
!skip7
		ROR
		ROR xplyd
		BCC !skip8
		ADC xplyc
!skip8
		ROR
		ROR xplyd
		BCC !skip9
		ADC xplyc
!skip9
	 	rts






;*****************************************************************************
; Simple Random Number
;*****************************************************************************
Random_Number:
	lda	#$11
	eor	Timera_2
	sta	Timerb_2
	
	lda	Timera_2+1
	eor	Timerb_2+1
	sta	Timerb_2+1
	clc
	adc	Timerb_2
	eor	Raster
	rts


;*****************************************************************************
;
; Function:		Simple player-2-sprite collision
; Description:	This does a simple center+radius collision check to the player
;				but can and should be heavily rewritten depending on the game.
;
;*****************************************************************************
PlayerYCenterOff	equ	8
PlayerXCenterOff	equ	8/2
PlayerWidth			equ	6/2
PlayerHeight		equ	6
PlayerCenterX		equ	Temp
PlayerCenterY		equ	Temp+1
CenterSubtraction	equ	Temp+2
DoSpriteCollision:
			clc
			lda	SpY					; Work out player center on Y
			adc	#PlayerYCenterOff	; by adding on a center offset
			sta	PlayerCenterY

			lda	SpXHi				; PlayerCenterX = (SpriteX/2)+(CenterOffset/2)
			lsr
			lda	SpXLo
			ror
			clc
			adc	#PlayerXCenterOff	; Center /2 because we've scaled back a bit
			sta	PlayerCenterX


			;
			; Now do actual player-2-sprite collision loop
			;
			ldx	#MaxSprites-1
!TestAll:
			lda	SpY,x				; Is this sprite active?
			beq	!SpriteNotActive

			;
			; First test collision on "X"
			;
			lda	SpXHi,x				; SpriteCenterX = (SpriteX/2)+(CenterXOffset/2)
			lsr
			lda	SpXLo,x
			ror
			clc
			adc	CenterXoffset,x		; SpriteCenterX-PlayerCenterX = CenterOffsetX			(already/2)
			sec
			sbc	PlayerCenterX
			bpl	!StoreX
			eor	#$ff				; Store ABS(SpriteCenterX-PlayerCenterX)
			adc	#1
!StoreX:	sta	CenterSubtraction

			clc						; Now get SpriteRadus+PlayerRadius = TotalRad
			lda	RadX,x
			adc	#PlayerWidth
			cmp	CenterSubtraction	; if(  TotalRad > CenterOffset ) then we've collided on Y
			bcc	!NoCollision


			;
			; If we get here, then we've collided on "X" so now try "Y"
			;
			lda	SpY,x
			clc
			adc	CenterYoffset,x		; SpriteCenter-PlayerCenter = CenterOffset
			sec						; (Player center could be offset by 1 to get rid of "sec")
			sbc	PlayerCenterY		; Subtract player center        
			bpl	!NotNeg1
			eor	#$ff				; abs(CenterOffset)			Carry is already clear
			adc	#1					
!NotNeg1:	sta	CenterSubtraction	; Store abs(CenterOffset)

			clc						; Now get SpriteRadus+PlayerRadius = TotalRad
			lda	RadY,x
			adc	#PlayerHeight
			cmp	CenterSubtraction	; if(  TotalRad > CenterOffset ) then we've collided on Y
			bcc	!NoCollision


			;
			; Do your collision code here!
			;
			lda	#0
			sta	SpY,x


!NoCollision:
!SpriteNotActive:
			dex
			bne	!TestAll 			; Dont do sprite "0" as thats the player!
			rts


;*****************************************************************************
;
; Function:		Simple player-2-sprite collision
; Description:	This does a simple center+radius collision check to the player
;				but can and should be heavily rewritten depending on the game.
;
;*****************************************************************************
Anim		equ	Temp

AnimateSprites:
			ldx	#MaxSprites
!AnimateAll	:
			lda	AnimLo,x			; Get the base address of the animation
			sta	Anim
			lda	AnimHi,x
			sta	Anim+1
			ldy	AnimFrame,x			; get the frame index. if negative, initialise the animation
			bpl	!NoReset
!ResetAnim
			ldy	#0					; Get the delay
			lda	(Anim),y
			sta	AnimDelay,x			; store it
			iny
			lda	(Anim),y
			sta	SpShape,x			; get the 1st graphic
			iny
			tya
			sta	AnimFrame,x			; store the current frame for the next itteration
			bne	!NextSprite
!NoReset:
			dec	AnimDelay,x			; decrement the delay
			bne	!NextSprite			; if not zero, then we're not done waiting yet.

			lda	(Anim),y			; Get the NEXT frame
			beq	!ResetAnim			; 0? end of anim, so reset 	(Depending on where your sprites are, this might have to change)
			sta	SpShape,x			
			iny						; move to next frame
			tya
			sta	AnimFrame,x
			ldy	#0					; get master delay
			lda	(Anim),y
			sta	AnimDelay,x			; and store...
!NextSprite
			dex						; animate ALL sprites...
			bpl	!AnimateAll			; even the player (sprite 0)
			rts

			;
			;	Delay,   Frame,Frame,Frame,   Termination (0)
			;
Animation1:	db	250,	EmptySpriteShape+2, 0
Animation2:	db	2,		EmptySpriteShape+1, EmptySpriteShape+1,EmptySpriteShape+1,EmptySpriteShape+1,EmptySpriteShape+2,	0




