;******************************************************************************************************************
;
;  								              C64 Framework V1.0
;
;									                 By 
;
;									            Mike Dailly 
;  								            (c) Copyright 2008
;                                          All Rights Reserved
;
;                                             mike@dailly.org
;	
; Terms of use:
;
;		This Source Code is given away free and you may do anything you like with it except
;		sell it. You should also seriously consider releasing the source to any game or application 
;		that uses any part of this program. Any new feature, addition or enhancement to any part of 
;		the framework must have its source code released for all to use. Such additions must also be 
;		be sent directly to me for possibly inclusion of the framework.
;
;		Credit must also be given to the original authors if any part of this source is used.
;
;		     Michael Dailly retains copyright of this program and its source+assets.
;
;
;	This program and its source were made to further the retro comunity, and breaking any of these conditions
;	renders all the years of work meaningless. So have fun with it, and then let others have fun with your work too!
;
;
;******************************************************************************************************************
DEBUG					equ	1				; Enable DEBUG code  (enables colour bars and anything else you'd like)
CHEAT					equ 0				; cheat mode (1=on)
FramesPerGameCycle		equ	0				; Number of VBlanks per game cycle (0=50fps, 1=25fps etc.)

;-------------------------------- Multiplexor -------------------------------------
;
; Sprites should be +1 to allow for the PLAYER sprite which appears at the
; START of the sprite array. This makes controling/animating/colliding with it
; simpler as in many cases it's just part of the normal loop.
; When changing this number remember to alter the SORT routine macros.
;
LinkedListSort			equ	0
SlowSort				equ	1
StackSort				equ	2

MaxSprites				equ	16+1			; Number of sprites to allocate 
SortMethod				equ	StackSort		; Which multiplexor sort do we use?
MINSPRITEY				equ	29				; MIN "Y" coordinate before its clipped
MAXSPRITEY				equ	$f9				; MAX "Y" coordinate before its clipped
SAFEMODE				equ	1				; Make sure bad rasters dont kill the miltiplexor

					if SAFEMODE = 1
sub_amount				equ	13				; Y Sprite position - "x" number of rasters for Multiplexor IRQ
					else
sub_amount				equ	10				; When not in SAFEMODE we can shave a few more rasters off and get closer to the last set of sprites
					endif
;
;------------------------------------------------------------------------------------




Timer_Seed1				equ	$2a05
Timer_Seed2				equ	$a34b


PanelRaster				equ	$da				; If your going to use a panel of any sort..



;
; Memory map.
;
CodeStart			equ	$0830			; START of code
CodeEnd				equ	$3fff			; END of code
VICStart			equ	$4000			; 2 screens at $4000 and $4400
HWScreen1			equ	$4000			; character screen 1
HWShape1			equ	HWScreen1+$3f8	; sprite shapes
HWScreen2			equ	$4400			; character screen 2
HWShape2			equ	HWScreen2+$3f8	; sprite shapes
EmptySprite			equ	$4800			; Empty sprite
EmptySpriteShape	equ	((EmptySprite-VICStart)/64)
SpriteData			equ	$4840			; Start of sprites
CharacterSet		equ	$7800			; Game character set

;************************************************************************************************************************************
;
; 			Macros
;
;************************************************************************************************************************************


; ------------------------------------
; Normal colour bar
; ------------------------------------
if	DEBUG=1
COLBAR	macro	
		lda	#\0
		sta	$d020
		endm
else
COLBAR	macro	

		endm
endif

; ------------------------------------
; IRQ colour bar ON
; ------------------------------------
if	DEBUG=1
COLBARON	macro	
		inc	$d020
		endm
else
COLBARON	macro	

		endm
endif

; ------------------------------------
; IRQ colour bar OFF
; ------------------------------------
if	DEBUG=1
COLBAROFF	macro	
		dec	$d020
		endm
else
COLBAROFF	macro	

		endm
endif



;
; This is a varibale space allocator
;
AllocAddr	set		0			; init
Alloc		macro
\0:			equ		AllocAddr
AllocAddr	set		AllocAddr+\1
			endm


;
; Handy macros for LONG branches
;

jne			macro				
			beq		*+5
			jmp		\0
			endm

jeq			macro				
			bne		*+5
			jmp		\0
			endm

jpl			macro				
			bmi		*+5
			jmp		\0
			endm

jmi			macro				
			bpl		*+5
			jmp		\0
			endm

jcc			macro				
			bcs		*+5
			jmp		\0
			endm

jcs			macro				
			bcc		*+5
			jmp		\0
			endm



;************************************************************************************************************************************
; XPLY routine
;
; enter : xplyc = xp 1 ; xplyd = xp 2
;
;		  XPLYC can NOT be zero!!
;
; exit  : A = ans hi / xplyc = ans lo 
; used  : a
;
; Totals:
; 119 worst
; 101 best
;************************************************************************************************************************************
XPLY_M	macro
		LDA #$00		; 2
		dec	xplyc		; 5  pre-decrement to offset a SET carry when adding "xplyc"

		lsr xplyd		; 5
		BCC !skip1		; 2
		ADC xplyc		; 3	=10
!skip1
		ROR				; 2
		ROR xplyd		; 5
		BCC !skip2		; 3 = 10 (*9=90 best)
		ADC xplyc		;
!skip2
		ROR				; 2
		ROR xplyd		; 5
		BCC !skip3		; 2
		ADC xplyc		; 3 = 12 (*8=96 worst)
!skip3
		ROR
		ROR xplyd
		BCC !skip4
		ADC xplyc
!skip4
		ROR
		ROR xplyd
		BCC !skip5
		ADC xplyc
!skip5
		ROR
		ROR xplyd
		BCC !skip6
		ADC xplyc
!skip6
		ROR
		ROR xplyd
		BCC !skip7
		ADC xplyc
!skip7
		ROR
		ROR xplyd
		BCC !skip8
		ADC xplyc
!skip8
		ROR
		ROR xplyd
		BCC !skip9
		ADC xplyc
!skip9
		endm

;************************************************************************************************************************************
;
;	Allocate/Free an object. 
;	Out:	X=spare slot or -1 for error
;
;	Best	29	-	OLD
;	Worst	134	-	OLD
;
;	Alloc - BEST/WORST	-	12/18
;	Free  - Best/Worst	-	24	  (constant)
;	
;
; Usage:-		FastAlloc	[x|y] ListAddress ObjectOnOff    -Specify the X or Y register that hold the object to free
;           	X or Y		= index register to use
;				ListAddress	=	Base of pre-filled stack
;				ObjectOnOff =	Value to clear/set when allocated/free'd
;
;************************************************************************************************************************************
FastAlloc	macro	
				ld\0	\1FreeIndex			; 3		ldx	FreeIndex
				lda		\1FreeList,\0		; 4		lda	FreeList
				bmi		!NoneFree			; 2		bmi	!NoneFree
				in\0						; 2		inx
				st\0	\1FreeIndex			; 3		stx	FreeIndex
!NoneFree:									;		tax
				ta\0						; 2		get next free bullet (or $ff for none left)
			endm

;************************************************************************************************************************************
;
; Usage:-		FastFree	[x|y] ListAddress ObjectOnOff
;           	X or Y		=	index register to use (and hold the object to free)
;				ListAddress	=	Base of pre-filled stack
;				ObjectOnOff =	Value to clear/set when allocated/free'd
;
;************************************************************************************************************************************
FastFree	macro	
				t\0a						; 2		txa
				ld\0	\1FreeIndex			; 3		ldx	FreeIndex
				de\0						; 2		dex
				st\0	\1FreeIndex			; 3		stx	FreeIndex
				sta		\1FreeList,\0		; 5		sta	FreeList,x
				ta\0						; 2		tax				-	restore index
				lda		#0					; 2		lda	#0
				sta		\2,\0				; 5		sta	BullInUse,x
			endm




DEBUGGER	equ		1

;************************************************************************************************************************************
;
; 				Hardware stuff
;
;************************************************************************************************************************************
IRQ_VECTOR		equ		$fffe


RasterHi		equ		$d011
RasterLo		equ		$d012
CharSetBase		equ		$d018
VideoBase		equ		$d018


BLANK			equ		$10
BMM				equ		$20
TEST			equ		$80

RVSOFF			equ		$80
PAL				equ		$40
FREEZE			equ		$20
MCM				equ		$10

;************************************************************************************************************************************
;
; 				Zero page variables
;
;************************************************************************************************************************************
AllocAddr		set	2								; Start of ZeroPage RAM

				Alloc	temp,15						; General temp variables
				Alloc	addr,2						; handy indirect address point
				Alloc	VBlank,1					; VBlank happened flag
				Alloc	xplyc,1
				Alloc	xplyd,1

				;
				; Sprite paths use these a lot... so its quicker if there here.
				; Since its zero page, it should be acessed with a ,x to get best speed
				; These can be moved to NON-Zero page if there is a lack of zero page 
				; space however it WILL affect the speed of the multiplexor sort.
				;
				Alloc	SpY,MaxSprites				; Sprite Y (in pixels)

; Multiplexor stuff
				if SortMethod=0
					Alloc	SPNext,MaxSprites+1			; Sort Linked list
					Alloc	xcount,1					; multiplexor SORT counter
				endif
				if SortMethod=1
					Alloc	yorder,MaxSprites+1
				endif
				if SortMethod=2
					Alloc	YOrder,MaxSprites+1			
					Alloc	SPTemp,1
				endif
				Alloc	SpriteIndex,1				; Multiplexor current sprite
				Alloc	HWIndex,1					; Multiplexor HW Sprite index

; Joystick and Keyboard
				Alloc	JoyStick,2					; Joystick! (and keys)
				Alloc	GameKeys,2					; Keys the game is interested in

; MMC64 file reader
				Alloc	memptr,2					; MMC Work data used for sector loading
				Alloc	NamePtr,2					; Pointer to current filename to load
				Alloc	MMCDestAdd,2				; LOAD address
				Alloc	MMCTemp,2					; General TEMP storage
				Alloc	Sector,3
				Alloc	CurrentCluster,2			; current cluster in search
				Alloc	SectorCount,1				; Sectors left in this cluster

				Message "ZPage End = ",AllocAddr



	        end
