;******************************************************************************************************************
;
;  								              C64 Framework V1.0
;
;									                 By 
;
;									            Mike Dailly 
;  								            (c) Copyright 2008
;                                          All Rights Reserved
;
;                                             mike@dailly.org
;	
; Terms of use:
;
;		This Source Code is given away free and you may do anything you like with it except
;		sell it. You should also seriously consider releasing the source to any game or application 
;		that uses any part of this program. Any new feature, addition or enhancement to any part of 
;		the framework must have its source code released for all to use. Such additions must also be 
;		be sent directly to me for possibly inclusion of the framework.
;
;		Credit must also be given to the original authors if any part of this source is used.
;
;		     Michael Dailly retains copyright of this program and its source+assets.
;
;
;	This program and its source were made to further the retro comunity, and breaking any of these conditions
;	renders all the years of work meaningless. So have fun with it, and then let others have fun with your work too!
;
;
;******************************************************************************************************************
	include "hardware.inc"      			; get header file. equates defines etc.
	include "Frame.inc"      				; get header file. equates defines etc.


	opt		prg
	opt		C64=Start						; Insert BASIC header, and jump to start of the code



		org	CodeStart
Start:
	message "start=", Start
		sei
		ldx	#$ff
		txs

		ldx	#0								; clear stack & zero page
		txa
!loop
		sta	$0100,x
		sta	$00,x
		dex
		bne	!loop


		; Fill top of stack for use in multiplexor sort.
		ldx	#$7f
		lda	#$ff
!loop2	sta	$0100,x
		dex
		bpl	!loop2
		lda	#0					; end of sort marker
		sta	$0180
		
;		jsr	Init_MMC64			; Init the MMC64
;
;		lda	#0
;		sta	MMCDestAdd			; Load a test file
;		lda	#4
;		sta	MMCDestAdd+1
;		lda	#Lo(TestFile)
;		ldy	#Hi(TestFile)
;		jsr	LoadFile


		lda	#$2f
		sta	roms_io
		lda	#$35
		sta	roms

		lda	#0
		ldy	#MAXSPRITES
!ClearTable:
		sta	IRQ_SpriteY,y
		dey
		bpl	!ClearTable


		; Clear screen
		lda	#0
		sta	Blank_Char
		lda	#6			; MCM "red"
		sta	Colour
		jsr	Cls


		; Set up timers
		lda	#$10
		sta	Timera_control_2
		sta	Timerb_Control_2
		lda	#Lo(0)
		sta	Timera_2
		lda	#Hi(Timer_Seed1)
		sta	Timera_2+1
		lda	#Lo(Timer_Seed2)
		sta	Timerb_2
		lda	#Hi(timer_Seed2)
		sta	Timerb_2+1
		lda	#%00000001
		sta	Timera_Control_2
		sta	Timerb_Control_2


		lda	#Lo(VBlank_irq)			; set hardware irq vector
		sta	hard_irq
		lda	#Hi(VBlank_irq)
		sta	hard_irq+1

		lda	#Lo(Crash_Hard)			; set hardware nmi vector
		sta	hard_nmi
		lda	#Hi(Crash_Hard)
		sta	hard_nmi+1
	
		lda	#raster_irqs			; Enable Raster Interupts
		sta	vic_irq_control
		sta	vic_mask

		lda	#0
		sta	Cia1+$0e
		sta	Cia1+$0f

		lda	#$fb					; raster line for irqs
		sta	raster
		lda	#$1b					; setlect top half of screen
		sta	raster_hi
		lda	#1
		sta	$d019

		lda	#-1						; All sprites ON
		sta	sprites_on
		lda	#0						; none xpanded
		sta	Xpand
		sta	Ypand
		sta	Sprite_Pri				; all on top


		; 2 locations in case you double buffer the screen
		lda	#$ff
		sta	HWShape1				; sprite shapes
		sta	HWShape1+1
		sta	HWShape1+2	
		sta	HWShape1+3	
		sta	HWShape1+4	
		sta	HWShape1+5	
		sta	HWShape1+6	
		sta	HWShape1+7	

		sta	HWShape2			; sprite shapes
		sta	HWShape2+1
		sta	HWShape2+2	
		sta	HWShape2+3	
		sta	HWShape2+4	
		sta	HWShape2+5	
		sta	HWShape2+6	
		sta	HWShape2+7	

		lda	#$ff
		sta	$d01c
		sta	VIC_sprites_on

		lda	#0
		sta	$d020
		sta	$d021

		lda	#$08
		sta	$D016

	
		lda	#2
		sta	$dd00		; VIC bank 1 ($c000-$cfff)


		cli

		jmp	MainLoopInit






;******************************************************************************************************************
; 
; IRQ's + NMI's
;
;******************************************************************************************************************
VBlank_Irq:
			COLBARON
			sta	VBLANK_EXIT_A+1
			stx	VBLANK_EXIT_X+1
			sty	VBLANK_EXIT_Y+1


			;
			; Do vertical blank and flip buffers if need be
			;
			; Screen 1 is FRONT buffer
			; CharSetAdd1 is Front charset
			;
			inc	FrameCounter
			LDA	VBlank				; main loop READY to flip?
			bne	NoFlip

			lda	#((((CharacterSet/256)/8)*2)&$0f) + (((HWScreen1-VICSTart)/$400)<<4)
			sta	$d018

			; Copy in player sprite stuff
			lda	SpY
			sta	$d001
			lda	SpXLo
			sta	$d000
			lda	SpXHi
			sta	$d010
			lda	SpShape
			sta	HWShape1
			sta	HWShape2
			lda	SpCol
			sta	$d027
			
			jsr Sort					; SORT sprites


			lda	#0
			sta	FrameCounter
			lda	#$ff
			sta	VBlank
NoFlip:
			lda	#med_gray
			sta	$d022
			lda	#dark_gray	
			sta	$d023

			lda	#white
			sta	$d025
			lda	#dark_gray
			sta	$d026


			; ----------------------------
			; Copy in first 7 sprites
			lda	$d010
			and	#$01					; keep player bit
			sta	$d010

			ldx	#0						; for the first 7 sprites, x+1 IS the sprite index
			stx	SpriteIndex
			ldy	#2						; start sprite index
!CopyAll:
			lda	IRQ_SpriteY,x
			beq	EndOfSpriteList
			sta	$d001,y

			lda	IRQ_SpriteX,x
			sta	$d000,y

			lda	$d010
			ora	IRQ_SpriteSig,x			; already masked for the correct bit!
			sta	$d010

			lda	IRQ_SpriteShape,x
			sta	HWShape1+1,x
			sta	HWShape2+1,x

			lda	IRQ_SpriteCol,x
			sta	$d027+1,x

			iny
			iny

			inx
			cpx	#7
			bne	!CopyAll
			; End of inital copy
			; --------------------------------------




			; --------------------------------------
			; Any MORE sprites to display?
			; If so setup the next IRQ
			lda	IRQ_SpriteY,x
			beq	NoMoreSprites
SetupNewIRQ
			stx	SpriteIndex
			ldy	#1
			sty	HWIndex				; IF theres more after this, then it'll start at index 1 again

			sec
			sbc	#sub_amount
			sta	$d012

			lda	#Lo(MultiPlexor_irq)	;set hardware irq vector
			sta	hard_irq
			lda	#Hi(MultiPlexor_irq)
			sta	hard_irq+1

			jmp	SkipCLearRest




EndOfSpriteList:
			;lda	#0					; A already 0
			sta	$d001,y
			iny
			iny
			cpy	#16
			bcc	EndOfSpriteList
NoMoreSprites:
			; If no rasters needed we need to setup the NEXT
			; IRQ - possibly a panel?
;			lda	#PanelRaster
;			sta	$D012
;			lda	#Lo(Panel_irq)		;set hardware irq vector
;			sta	hard_irq
;			lda	#Hi(Panel_irq)
;			sta	hard_irq+1

SkipCLearRest:
			;End of sprite copy
			;------------------------------



Exit_VBlank
			inc	vic_irq_control
			lda	irq_control					; reset cia1 irq

ExitIRQ:
VBLANK_EXIT_A
			lda	#$00
VBLANK_EXIT_X
			ldx	#$00
VBLANK_EXIT_Y
			ldy	#$00
			COLBAROFF
Crash_Hard:					; Hardware nmi jumps/returns here
			rti				; when game has crashed



;
; Cut these tables down depending on how many sprites you enable
;
SigBitTable:	
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;
			db	%00100000	;
			db	%01000000	;
			db	%10000000	;
			db	%00000010	;
			db	%00000100	;
			db	%00001000	;
			db	%00010000	;


SigBitTableMask:	
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;
			db	%11011111	;
			db	%10111111	;
			db	%01111111	;
			db	%11111101	;
			db	%11111011	;
			db	%11110111	;
			db	%11101111	;



;
;	\0 = sprite number (1 to 7)
;	\1 = x+y offset (2,4,6,8,10,12,14)
;	\2 = NEXT sprite
;	\3 = "b" for bcc or "j" for jcc
;
SetUpSprite	macro
DoSprite\0:
			cpy	 #\0					; is this the Hardware sprite we're after?
			bne	 NextHWSprite\0			; not THIS sprite ->jump to next
SkipCPY\0:
			lda	 IRQ_SpriteX,x				; Store the sprites "X" coordinate in the
			sta	 $d000+\1				; VIC hardware sprite register

			lda	 $d010					; get all sprite significant bits
			and	 #~(1<<\0)				; mask OFF this one
			ora	 IRQ_SpriteSig,x		; already set for the correct bit!
			sta	 $d010					; so we OR in this sprite and store it!

			lda	 IRQ_SpriteY,x			; Get sprite Y coordinate
			sta	 $d001+\1				; store in VIC hardware register

			lda	 IRQ_SpriteShape,x		; get the sprite graphic
			sta	 HWShape1+\0			; Since we have a double buffered screen, 
			sta	 HWShape2+\0			; we store it in BOTH locations.

			lda	 IRQ_SpriteCol,x		; Get sprite colour
			sta	 $d027+\0				; store in VIC hardware

			inx							; next sprite in list
			ldy	 #\2					; next hardware sprite

			lda	 IRQ_SpriteY,x			; Get sprite Y
			jeq	 EndOfSpriteList2		; 0? If so all finished
			sbc	 $d012
			cmp	 #7						; do we HAVE enough time for another IRQ? (magic number)
			\3cc SkipCPY\2				; if not do it now!


            ;
			; If we've lots of raster time until the next sprite
			; but the sprite we will be copying over was already drawn
			; by the VIC, then just copy it over NOW!
			;
			;sec						; Carry is AREADY set...
			lda	 $d012					; Get the current raster
			sbc	 #22					; -22 gets us the last "safe" sprite
			sbc	 $d001+(\2*2)			; location. Now subtract the next sprite Y
			\3cs SkipCPY\2				; is > last sprite, then setup NOW!

			;
			; If its further down the screen, and the current sprite
			; is still being drawn, then setup a new IRQ just above the 
			; sprite to handle it
			;
!GetNext:
			; Disable slow code once everything is working properly
			if SAFEMODE = 1
			;
			; If sprites bounce around randomly, you have to take extra care to
			; avoid illegal IRQ's being kicked off (ones behind you etc.)
			;
			; THIS DOESN'T REALLY WORK PROPERLY... BUT IT HELPS STOP OVERLAPPING IRQ'S.
			; A PROPER FIX WOULD BE PRETTY SLOW, AND ITS BETTER TO JUST AVOID NEEDING IT!
			;
			;sec						; if we can't copy now, then we set up
			lda	 IRQ_SpriteY,x			; a new RASTER IRQ based on the next sprite
			jeq	 EndOfSpriteList2		; 0? If so all finished
			sbc	 #sub_amount-1			; Next make sure theres enough time
			cmp	 $d012					; until the next iterrupt, if not, disguard 
			bcs	 !ExitIRQ				; this sprite and move onto next.
			inx							; This stops "bad" raster interrupts.
			bne	 !GetNext				; and loop back until we've checked all remaining 
!ExitIRQ:	adc	 #2						; sprites. Add 1+carry get back to sub_amount 
			jmp	 EndMultiIRQ

			else

			;
			; If sprites will NEVER get out of control, you can do this instead
			;
			;sec						; if we can't copy now, then we set up
			lda	 IRQ_SpriteY,x			; a new RASTER IRQ based on the next sprite
			sbc	 #sub_amount-1			; Next make sure theres enough time
			jmp	 EndMultiIRQ

			endif

NextHWSprite\0:
			endm



;******************************************************************************
; Actual multiplexor....
MultiPlexor_irq:
			COLBARON
			sta	EXIT_MultiPlex_A+1
			stx	EXIT_MultiPlex_X+1
			sty	EXIT_MultiPlex_Y+1
			cld

			

			ldx	SpriteIndex
			ldy	HWIndex
StartAgain
			SetUpSprite	1,	2,	2,	b
			SetUpSprite	2,	4,	3,	b
			SetUpSprite	3,	6,	4,	b
			SetUpSprite	4,	8,	5,	b
			SetUpSprite	5,	10,	6,	b
			SetUpSprite	6,	12,	7,	b
			SetUpSprite	7,	14,	1,	j

			; --------------------------------------
			; End of IRQ
			;
EndMultiIRQ
			sta	$D012					; and then this starts all over again!
			stx	SpriteIndex
			sty	HWIndex
Exit_The_IRQ
			inc	vic_irq_control
			lda	irq_control		;reset cia1 irq

EXIT_MultiPlex_A
			lda	#$00
EXIT_MultiPlex_X
			ldx	#$00
EXIT_MultiPlex_Y
			ldy	#$00
			COLBAROFF
			rti				; when game has crashed


			;
			; End of multiplexor IRQ's 
			; Setup panel IRQ
			;
EndOfSpriteList2:
			lda	#$fb
			sta	$D012

			lda	#Lo(VBlank_Irq)		;set hardware irq vector
			sta	hard_irq
			lda	#Hi(VBlank_Irq)
			sta	hard_irq+1
			jmp	Exit_The_IRQ



;******************************************************************************
; CLS	: clears screens with a balnk_char and a colour

CLS			ldx	#00							;set up index
clear_loop	lda	#$00
			sta	HWScreen1,x
			sta	HWScreen2,x
			sta	HWScreen1+$0100,x
			sta	HWScreen2+$0100,x
			sta	HWScreen1+$0200,x
			sta	HWScreen2+$0200,x
			sta	HWScreen1+$0300,x
			sta	HWScreen2+$0300,x
_clscolour	lda	#$00						;get colour to clear screen with
			sta	colour_screen,x
			sta	colour_screen+$0100,x
			sta	colour_screen+$0200,x
			sta	colour_screen+$0300,x
			dex
			bne	clear_loop
			rts


blank_char	equ		clear_loop+1
colour		equ		_clscolour+1



;*****************************************************************************
; SORT: sort out by y cord and store in:
;
; if y-cord = $00 then sprite is NOT active (or off screen)
;
;*****************************************************************************
			if	SortMethod = 0
Sort:
			ldx	#MAXSPRITES-1
!FindFirst:
			lda	SpY,x
			cmp	#MINSPRITEY			; <MINSPRITEY & >$MAXSPRITEY
			bcs	!FoundFirst			; means we clip the sprite
!BackHere
			dex
			bne	!FindFirst
			lda	#0
			sta	IRQ_SpriteY
			rts						; NO sprites on!

!FoundFirst
			cmp	#MAXSPRITEY			; if >=MAXSPRITEY then sprite is actually 
			bcs	!BackHere

			;
			; Found 1st sprite
			;
			lda	#-1					; set first active as last in list
			sta	SPNext+1,x
			stx	SPNext				; and set first active as FIRST
			dex						; and move on one - we dont need to do the 1st one
			beq	!AllDone
!SortAll
			lda	SpY,x				; Get "Y"
			cmp	#MINSPRITEY
			beq	!DoNext
			cmp	#MAXSPRITEY			; if >=219 then sprite is actually 
			bcs	!DoNext


			stx	xcount				; Sprite number we're inserting
			ldx	#-1					; pLast
			ldy	SPNext				; pCurrent
			;
			; X and Y take turns about at being pCurrent and pLast...
			; The first itteration X=pLast, Y=pCurrent. The second is reversed.
			;
!FindSpace:
			cmp	SpY,y
			bcc	!InsertHere
			ldx	SPNext+1,y			; get next
			bmi	!InsertHere2		; Not end of list... so keep going!
			cmp	SpY,x
			bcc	!InsertHere2
			ldy	SPNext+1,x			; get next
			bpl	!FindSPace			; Not end of list... so keep going!
!InsertHere
			inx						; allow for -1 when head of list
			lda	xcount				; Set last->pNext to be this one
			sta	SPNext,x
			tax						; Now move to the new entry
			sty	SPNext+1,x
			dex						; "X" already holds xcount!
			bne	!SortAll
			beq	!AllDone

!InsertHere2
			lda	xcount				; Set last->pNext to be this one
			sta	SPNext+1,y			; "Y" can never be negative... So use +1 again
			tay						; Now move to the new entry
			stx	SPNext+1,y
			tax						; A already holds the current number
!DoNext
			dex
			bne	!SortAll
!AllDone

			;
			; Now Copy into IRQ buffer...
			;
			ldy	#0
			ldx	SPNext
!MoreToCopy:
			lda	SpY,x			; store y cord
			sta	IRQ_SpriteY,y		

			lda	SpXLo,x			; copy x cord
			sta	IRQ_SpriteX,y
			lda	SpxHi,x			; copy sig bit
			beq	!Store
			lda	SigBitTable,y
!Store:		sta	IRQ_SpriteSig,y

			lda	SpShape,x
			sta	IRQ_SpriteShape,y

			lda	SpCol,x
			sta	IRQ_SpriteCol,y

			iny
			lda	SPNext+1,x			; Follow linked list
			tax
			bpl	!MoreToCopy

			lda	#$00				; set last sprite flag
			sta	IRQ_SpriteY,y
			rts
			endif					; end sortmethod=1

;*****************************************************************************
; OLD Sort macro
;*****************************************************************************
sort_mac	macro	;index_in
			cmp		yorder+\0			; is this value lower than the
			bcc		!no_high			; current lowest value if not try next
			ldx		#\0					; if it is then set current lowest to
			lda		yorder+\0			; value found, and y to index.
!no_high
			endm

;*****************************************************************************
; OLD sort. In some rare cases this sort may turn out to be quicker
; in which case you can enable it by changing the SortMethod equate 
; to be 1 rather than 0.
;*****************************************************************************
			if	SortMethod=1
Sort:
			ldx	#MAXSPRITES-1
!copy:
			lda	SpY,x
			cmp	#MAXSPRITEY		; if >=219 then sprite is actually 
			bcs	!OffScreen		; OFF the bottom of the screen!
			cmp	#MINSPRITEY		; if <29 then sprite is actually 
			bcs	!OnScreen		; OFF screen!
!OffScreen:
			lda	#-1			
!OnScreen:
			sta	yorder,x
			dex
			bpl	!copy


			ldy	#0					; set at first sprite to be displayed

			;
			;	Comment in/out the "sort_mc" lines below depending on the 
			;   number of active sprites you want.
			;
testloop1	lda	#MAXSPRITEY			; set current lowest number
			sort_mac	1			; now look through all 16 for the
			sort_mac	2			; lowest number and pick it up
			sort_mac	3			; then wipe low number with a high
			sort_mac	4			; number ($ff) so sort will skip over
			sort_mac	5			; it
			sort_mac	6
			sort_mac	7			; index 0 is the master path
			sort_mac	8
			sort_mac	9
			sort_mac	10
			sort_mac	11
			sort_mac	12
			sort_mac	13
			sort_mac	14
			sort_mac	15
			sort_mac	16
			;sort_mac	17
			;sort_mac	18	  
			;sort_mac	19	  		
			;sort_mac	20			
			;sort_mac	21			
			;sort_mac	22
			;sort_mac	23			
			;sort_mac	24
			;sort_mac	25
			;sort_mac	26
			;sort_mac	27
			;sort_mac	28
			;sort_mac	29
			;sort_mac	30
			;sort_mac	31
			;sort_mac	32
			;sort_mac	33
			;sort_mac	34
			;sort_mac	35
			;sort_mac	36
			;sort_mac	37
			;sort_mac	38
			;sort_mac	39
			;sort_mac	40
			;sort_mac	41
			;sort_mac	42
no_higher
			cmp	#MAXSPRITEY			; is the lowest value still $f9
			beq	no_more_sprites
Sorted
			lda	#-1
			sta	yorder,x			; clear this slot

			; ---------------------------------------
			; Now copy sprite data over.
			; This also double buffers it for us
			lda	SpY,x
			sta	IRQ_SpriteY,y

			lda	SpXLo,x
			sta	IRQ_SpriteX,y
			lda	SpxHi,x
			beq	!Skip
			lda	SigBitTable,y			; Get the correct SIG bit "now",
!Skip		sta	IRQ_SpriteSig,y			; as its quicker than later!

			lda	SpShape,x				; Sprite SHAPE
			sta	IRQ_SpriteShape,y

			lda	SpCol,x
			sta	IRQ_SpriteCol,y
SkipCopy
			iny
			cpy	#MaxSprites
			jne	testloop1

no_more_sprites
			lda	#$00				; set last sprite flag
			sta	IRQ_SpriteY,y

			rts
			endif					; end sortmethod=1






; *****************************************************
; Add sprites to the bucket via the stack
; *****************************************************
StackInsert	macro
			lda	SpY+\0	 		; Get "Y" coords
			beq	!ignore			; Active? If not ignore it....
			lsr
			tax
			txs
!Search:	pla 				; Get sprite stored here, if negative ($80) then slot empty. if >0 then sprite index of 0-31.
			bpl	!Search			; Keep looking down until we find an empty slot!
			lda	#\0
			pha
!ignore			
			endm
		

; *****************************************************
; Search for sprites on the stack...
; *****************************************************
StackSearch	macro
!Search:	pla					; Loop and get next sprite in list
			bmi	!Search 		; if $ff then haven't found one yet
			\1eq FinishedSort	; if we hit a 0, then we're done!
	
			sta	YOrder,x		; Store found ID in order table
			inx					; next order table element.
			tya					; clear used slot by poutting $ff there.
			pha 				; move pointer past used slot (slightly quicker than leaving it

			pla 				; to the pla/BMI branch, since we KNOW its not the one we're after)
			endm




			if	SortMethod=2
; **********************************************************************************************************
; Armalyte Sort: enable by setting the sort to method 2
; **********************************************************************************************************
Sort:
			tsx 
			stx SPTemp			; store current stack pointer
  
  			;
			; Comment in/out the statements below depending on how
			; many sprites you've enabled
			;
			;StackInsert	42		
			;StackInsert	41		
			;StackInsert	40		
			;StackInsert	39		
			;StackInsert	38		
			;StackInsert	37		
			;StackInsert	36		
			;StackInsert	35		
			;StackInsert	34		
			;StackInsert	33		
			;StackInsert	32		
			;StackInsert	31		
			;StackInsert	30		
			;StackInsert	29		
			;StackInsert	28		
			;StackInsert	27		
			;StackInsert	26		
			;StackInsert	25		
			;StackInsert	24		
			;StackInsert	23		
			;StackInsert	22		
			;StackInsert	21		
			;StackInsert	20		
			;StackInsert	19		
			;StackInsert	18
			;StackInsert	17
			StackInsert	16
			StackInsert	15
			StackInsert	14
			StackInsert	13
			StackInsert	12
			StackInsert	11
			StackInsert	10
			StackInsert	9
			StackInsert	8
			StackInsert	7
			StackInsert	6
			StackInsert	5
			StackInsert	4
			StackInsert	3
			StackInsert	2
			StackInsert	1


			;
			; Now lift out values from the stack
			;
			LDX #24 			; anything above 29 is OFF SCREEN!!! 
			TXS 
			LDY #$FF
			ldx	#0 

			;StackSearch		42,j
			;StackSearch		41,j
			;StackSearch		40,j
			;StackSearch		39,j
			;StackSearch		38,j
			;StackSearch		37,j
			;StackSearch		36,j
			;StackSearch		35,j
			;StackSearch		34,j
			;StackSearch		33,j
			;StackSearch		32,j
			;StackSearch		31,j
			;StackSearch		30,j
			;StackSearch		29,j
			;StackSearch		28,j
			;StackSearch		27,j
			;StackSearch		26,j
			;StackSearch		25,j
			;StackSearch		24,j
			;StackSearch		23,j
			;StackSearch		22,j
			;StackSearch		21,j
			;StackSearch		20,j
			;StackSearch		19,j
			;StackSearch		18,j
			;StackSearch		17,j
			StackSearch		16	,j
			StackSearch		15	,j
			StackSearch		14	,j
			StackSearch		13	,j
			StackSearch		12	,j
			StackSearch		11	,b
			StackSearch		10	,b
			StackSearch		9	,b
			StackSearch		8	,b
			StackSearch		7	,b
			StackSearch		6	,b
			StackSearch		5	,b
			StackSearch		4	,b
			StackSearch		3	,b
			StackSearch		2	,b
			StackSearch		1	,b
			lda	#0
FinishedSort:			
			sta	YOrder,x

			LDX SpTemp
			TXS 



			;
			; Now Copy into IRQ buffer...
			;
			ldy	#0
!MoreToCopy:
			ldx	YOrder,y
			beq	!AllDone

			lda	SpY,x				; Copy "Y" into backup buffer
			sta	IRQ_SpriteY,y		

			lda	SpXLo,x				; copy x cord
			sta	IRQ_SpriteX,y
			lda	SpxHi,x				; copy sig bit
			beq	!Store
			lda	SigBitTable,y
!Store:		sta	IRQ_SpriteSig,y

			lda	SpShape,x			; Copy SHAPE
			sta	IRQ_SpriteShape,y

			lda	SpCol,x				; Copy Colour
			sta	IRQ_SpriteCol,y
			iny
			jmp	!MoreToCopy

!AllDone:
			lda	#$00				; set last sprite flag
			sta	IRQ_SpriteY,y

			rts
			endif					; end sortmethod=2


;*****************************************************************************
;
; Scans the joystick AND keyboard....
;
;*****************************************************************************
JOY_UP				equ		1
JOY_DOWN			equ		2
JOY_LEFT			equ		4
JOY_RIGHT			equ		8
JOY_FIRE			equ		16

KEY_F1				equ		1
KEY_F3				equ		2
KEY_F5				equ		4
KEY_F7				equ		8
KEY_LEFT_SHIFT		equ		16
KEY_RUNSTOP			equ		32

ScanKeys:
			; Read Joystick 1
			lda	joy_1
			and	#$1f
			eor	#$1f
			sta	Joystick
			; Read Joystick 2 Mix with Joy 1 so that port doesn't matter
			lda	joy_2
			and	#$1f
			eor	#$1f
			ora	Joystick
			sta	Joystick


			;
			; Now scan for all the GAME keys we're interested in. 
			; Key BITS are then set in the GameKeys variable.
			; LEFTSHIFT is a good pause key because of SHIFT_LOCK - it means you dont have to have an ON/OFF
			;
			ldy	#0
			sty	GameKeys
			sty	GameKeys+1

			dey
!ScanAllKeys
			iny
			lda	WriteValue,y
			beq	!ScanFinished
			sta	$dc00
			lda	$dc01
			and	ReadMask,y
			bne	!ScanAllKeys
			ldx	KeyIndex,y
			lda	KeyValue,y
			ora	GameKeys,x
			sta	GameKeys,x
			jmp	!ScanAllKeys
!ScanFinished:
			rts

WriteValue	db	write_f1, 	write_f3,	write_f5,	write_f7,	write_left_shift,	write_STOP, 0
ReadMask	db	read_f1,	read_f3,	read_f5,	read_f7,	read_left_shift,	read_STOP
KeyIndex	db	0,			0,			0,			0,			0,					0
KeyValue	db	%00000001,	%00000010,	%00000100,	%00001000,	%00010000,			%00100000


;*****************************************************************************
;
; Include Game files
;
;*****************************************************************************
			include	"main.s"
			include	"mmc64.s"
			include	"data.s"

EndOfCode:
		message	"Code Space Left=", CodeEnd-EndOfCode




