;******************************************************************************************************************
;
;  								              C64 Framework V1.0
;
;									                 By 
;
;									            Mike Dailly 
;  								            (c) Copyright 2008
;                                          All Rights Reserved
;
;                                             mike@dailly.org
;	
; Terms of use:
;
;		This Source Code is given away free and you may do anything you like with it except
;		sell it. You should also seriously consider releasing the source to any game or application 
;		that uses any part of this program. Any new feature, addition or enhancement to any part of 
;		the framework must have its source code released for all to use. Such additions must also be 
;		be sent directly to me for possibly inclusion of the framework.
;
;		Credit must also be given to the original authors if any part of this source is used.
;
;		     Michael Dailly retains copyright of this program and its source+assets.
;
;
;	This program and its source were made to further the retro comunity, and breaking any of these conditions
;	renders all the years of work meaningless. So have fun with it, and then let others have fun with your work too!
;
;
;******************************************************************************************************************

;
;		Vic,Rom,Kernal,Cia chip lables
;


	include	"key_tables.inc"

kernal_rom 				equ	%00000010  	; kernal rom i/o bit
basic_rom 				equ	%00000001  	; basic rom i/o bit
vic_chip_out			equ	$34			; value needed to bank out vic chip
tape_motor				equ	%00100000  	; tape moter bit (0=on)
char_rom 				equ	%00000100	; char rom bit (0=in)
raster_irqs				equ	%00000001	; raster irq bit

hard_irq				equ	$fffe
hard_nmi				equ	$fffc

Vic_Out					equ	$34				; page vic OUT
Vic_In					equ	$35				; page vic IN


;Useful KERNAL routines
ACPTR					equ	$FFA5	; input byte from serial port
CHKIN					equ	$FFC6	; open channel for input
CHKOUT					equ	$FFC9	; open channel for output
CHRIN					equ	$FFCF	; input byte from channel
CHROUT					equ	$FFD2	; output byte to channel
CIOUT					equ	$FFA8	; outputbyte to serial port
CINT					equ	$FF81	; init screen editor
CLALL					equ	$FFE7	; close all channels and files
CLOSE					equ	$FFC3	; close a specific logical file
CLRCHN					equ	$FFCC	; close input and output channels
GETIN					equ	$FFE4	; get char from keyboard buffer
IOBASE					equ	$FFF3	; returns base address of I/O devices
IOINIT					equ	$FF84	; init Input/Output
LISTEN					equ	$FFB1	; command devices on serial bus 2 listen
LOAD					equ	$FFD5	; load ram from device
MEMBOT					equ	$FF9C	; read/set the bottom of memory
MEMTOP					equ	$FF9C	; read/set the top of memory
OPEN					equ	$FFC0	; open a logical file
PLOT					equ	$FFF0	; read/set x,y cursos position
RAMTAS					equ	$FF87	; init RAM,alloc tape buff,set scr $0400
RDTIM					equ	$FFDE	; read real time clock
READST					equ	$FFB7	; read I/O status word
RESTORE					equ	$FF8A	; restore default I/O vectors
SAVE					equ	$FFD8	; save RAM to device
Scankey					equ	$ff9f	; scan keyboard
SECOND					equ	$FF93	; send secondary address to listen
SETLFS					equ	$FFBA	; set logical, 1st and 2nd addressess
SETMSG					equ	$FF90	; control KERNAL messages
SETNAM					equ	$FFBD	; set file name
SETTIM					equ	$FFDB	; set real time clock
SETTMO					equ	$FFA2	; set timeout on serial bus
TALK					equ	$FFB4	; command serial bus device to TALK
TKSA					equ	$FF96	; send secondary device after TALK
UDTIM					equ	$FFEA	; increment real time clock
UNLSN					equ	$FFAE	; command serial bus to UNLISTEN
UNTLK					equ	$FFAB	; command serial bus to UNTALK
VECTOR					equ	$FF8D	; read/set vectored I/O

KeyLog					equ	$028f	; scan key indirect jump
Fnlen					equ	$b7
Tmpc					equ	$97
Status					equ	$90
MsgFlg					equ	$9d	;System messages (on/off)

;************* VIC Display Chip, sprites, scroll, rasters, etc.... ************
vic						equ	$d000
sprite_x				equ	vic	     ;1st sprite xcord
sprite_y				equ	vic+1	 ;1st sprite ycord
sprite_sig				equ	vic+$10	 ;all sprite sig bits
raster_hi				equ	vic+$11	 ;raster bit
screen_on				equ	vic+$11	 ;screen on reg
screen_off				equ	screen_on
ecm						equ	vic+$11	 ;extended colour mode
y_smooth_scroll			equ	vic+$11	 ;hardware smooth scroll
raster					equ	vic+$12	 ;raster reg
sprites_on				equ	vic+$15	 ;Sprites on/off switch
VIC_sprites_on			equ	vic+$15	 ;Sprites on/off switch
x_smooth_scroll			equ	vic+$16	 ;smooth scroll reg (on x)
multi_colour			equ	vic+$16	 ;multi colour char mode
Ypand					equ	vic+$17	 ;Xpand sprites in y direction
char_shape				equ	vic+$18	 ;char set position
screen_pos				equ	vic+$18	 ;screen position
vic_irq_control			equ	vic+$19	 ;vic irq control reg
vic_mask				equ	vic+$1a	 ;irq mask
sprite_pri				equ	vic+$1b	 ;sprite to background priority
sprite_multi_colour		equ	vic+$1c	 ;sprite multi colour mode
Xpand					equ	vic+$1d	 ;Xpand sprites in x direction
boarder_colour			equ	vic+$20	 ;boarder colour (0-15)
screen_colour			equ	vic+$21	 ;background/screen colour(0-15)
multi_colour_1			equ	vic+$22	 ;char multi colour #1    (0-15)
ecm_colour_1			equ	vic+$22	 ;char extended colour #1 (0-15)
multi_colour_2			equ	vic+$23	 ;char multi colour #2    (0-15)
ecm_colour_2			equ	vic+$23	 ;char extended colour #2 (0-15) 
ecm_colour_3			equ	vic+$24	 ;char extended colour #3 (0-15)
sprite_multi_colour_1 	equ	vic+$25	 ;sprite multi colour #1  (0-15)
sprite_multi_colour_2 	equ	vic+$26	 ;sprite multi colour #2  (0-15)
sprite_colour			equ	vic+$27	 ;sprite colours          (0-15)


;********************** Colour Screen data ***********************************
colour_screen			equ	$d800		; colour display screen
black					equ	$00	 	 		; colour codes/pokes/values
white					equ	$01
red						equ	$02
cyan					equ	$03
purple					equ	$04
green					equ	$05
blue					equ	$06
yellow					equ	$07
orange					equ	$08
brown					equ	$09
light_red				equ	$0a
dark_gray				equ	$0b
med_gray				equ	$0c
light_green				equ	$0d
light_blue				equ	$0e
light_gray				equ	$0f


;*********************** IRQ Timers etc.... (CIA#1) **************************
cia1					equ	$dc00	 ;cia#1 chip start
joy_1					equ	cia1	 ;joystick port 1
key_write				equ	joy_1	 ;keyboard write row.
joy_2					equ	cia1+1	 ;joystick port 2
key_read				equ	joy_2	 ;keyboard read column (gives key)
ddr_porta_1				equ	cia1+2	 ;joystick 1(porta) data direction reg
ddr_portb_1				equ	cia1+3	 ;joystick 2(portb) data direction reg
timera					equ	cia1+4	 ;timer a (2 bytes lo-hi)
timerb					equ	cia1+6	 ;timer b (2 bytes lo-hi)
clock					equ	cia1+8	 ;time of day clock
serial_io				equ	$dc0c	 ;serial i/o data shift reg
irq_control				equ	cia1+$0d ;cia#1 irq controler
timera_control			equ	cia1+$0e ;timer a control reg
timerb_control			equ	cia1+$0f ;timer b control reg

;********************** NMI Timers etc.... (CIA#2) ***************************
cia2					equ	$dd00	 ;cia#2 chip start
porta					equ	cia2	 ;user port
portb					equ	cia2+1	 ;user port
ddr_porta_2				equ	cia2+2	 ;user port data direction reg
ddr_portb_2				equ	cia2+3	 ;user port data direction reg
timera_2				equ	cia2+4	 ;timer a (2 bytes lo-hi)
timerb_2				equ	cia2+6	 ;timer b (2 bytes lo-hi)
nmi_control				equ	cia2+$0d ;cia#2 nmi control reg
timera_control_2		equ	cia2+$0e ;timer a control reg
timerb_control_2		equ	cia2+$0f ;timer b control reg



;******************************** MMC64  *************************************
;
;	$8000 - $9fff	MMC64 BIOS rom (*)(**)
;	Rom is active when bit 0 of $DF11 and the flash jumper is not set
;	Rom is write enabled when bit 4 of $DF11 is set
;
ClockPort_Control		equ	$de01 	 ; Clockport On/Off
									 ;	(or $DF21) (w) bit 0: 0 = disable clock port, 1 = enable clockport
ClockPortBase			equ	$d002	 ; Clockport IO	(R/W)
									 ;	$de02-$de0f or $df22-$df2f.
									 ;	(location depends on bit 3 of $DF11)


MMC64_SPI				equ	$df10	 ; MMC64 SPI transfer register 
									 ;	byte written is sent to the card & response from the card is read here
MMC64_Control			equ	$df11	 ; MMC64 control register.
									 ; 	bit 0:	0 = MMC64 BIOS active, 1 = external ROM active
									 ;	bit 1:	0 = card selected, 1 = card not selected
									 ;	bit 2:	0 = 250khz transfer, 1 = 8mhz transfer
									 ;	bit 3:	0 = clock port @ $DE00, 1 = clock port @ $DF20
									 ;	bit 4:	0 = normal Operation, 1 = flash mode (*)
									 ;	bit 5:	0 = allow external rom when BIOS is disabled , 1 = disable external ROM
									 ;	bit 6:	0 = SPI write trigger mode, 1 = SPI read trigger mode
									 ;	bit 7:	0 = MMC64 is active, 1 = MMC64 is completely disabled (**)
									 ;	(*) bit can only be programmed when flash jumper is set
									 ;	(**) bit can only be modified after unlocking
MMC64_Status			equ	$d012	 ; MMC64 Status Register
									 ;	bit 0:	0 = SPI ready, 1 = SPI busy
									 ;	bit 1:	external GAME line
									 ;	bit 2:	external EXROM line
									 ;	bit 3:	0 = card inserted, 1 = no card inserted
									 ;	bit 4:  0 = card write enabled, 1 = card write disabled
									 ;	bit 5:	0 = flash jumper not set, 1 = flash jumper set 
MMC64_ID				equ	$d013	 ; MMC64 identification register
									 ;	(R)	#$64 when bit 1 of $DF11 is 0
									 ;		#$01 when bit 1 of $DF11 is 1 and REV A hardware is used
									 ;		#$02 when bit 1 of $DF11 is 1 and REV B hardware is used
									 ;		
									 ;	(W)	write #$55 & #$AA into this register to unlock bit 7 of $DF11
									 ;		write #$0A $ #$1C into this register to re-enable MMC64 hardware
									 

;*********************** zero page ROM locations...... ***********************
roms					equ	$01
r6510					equ	roms
roms_io					equ	$00
d6510					equ	roms_io
current_key				equ	$c5	 		;current key pressed (rom routine)
la						equ	$b8	 		;current logical file number
sa						equ	$b9	 		;current secondary address
fa						equ	$ba	 		;current device number
fnadr					equ	$bb	 		;pointer: current file name

stack					equ	$0100	 	;system stack (length = $0100)

mode					equ	$0291	 	;disable/enable shift keys (0=disable)
pal_ntsc				equ	$02a6	 	;flag: 0=ntsc, 1=pal



