;******************************************************************************************************************
;
;  								              C64 Framework V1.0
;
;									                 By 
;
;									            Mike Dailly 
;  								            (c) Copyright 2008
;                                          All Rights Reserved
;
;                                             mike@dailly.org
;	
; Terms of use:
;
;		This Source Code is given away free and you may do anything you like with it except
;		sell it. You should also seriously consider releasing the source to any game or application 
;		that uses any part of this program. Any new feature, addition or enhancement to any part of 
;		the framework must have its source code released for all to use. Such additions must also be 
;		be sent directly to me for possibly inclusion of the framework.
;
;		Credit must also be given to the original authors if any part of this source is used.
;
;		     Michael Dailly retains copyright of this program and its source+assets.
;
;
;	This program and its source were made to further the retro comunity, and breaking any of these conditions
;	renders all the years of work meaningless. So have fun with it, and then let others have fun with your work too!
;
;
;******************************************************************************************************************
MainLoopInit:
			jsr  Init


MainLoop	lda	#$00				; wait VBlank
			sta	VBlank				; IRQ will change this from a $00 
WaitVblank	lda	VBlank				; to $ff so we just need to wait...
			beq	WaitVblank

!DoPause
			jsr	ScanKeys            ; scan joystick/keys...


			lda	GameKeys
			and	#KEY_LEFT_SHIFT
			bne	!DoPause

			;
			; Your code here....
			;
			jsr	ProcessKeys
			jsr	MovePlayer
			jsr	MoveSprites
			jsr	AnimateSprites
			jsr	DoCollision


WaitFrames:
			lda	FrameCounter			; Wait for the desired number of frames to pass
			cmp	#FramesPerGameCycle		; 0=50FPS, 1=25fps
			bcc	WaitFrames
			jmp	MainLoop





;************************************************************************************************************************
;
;   Name    :  Init
;   Function:  Init the main loop
;
;************************************************************************************************************************
SpriteX	dw	0
SpriteY	db	0
Init:
			;
			; Create some test sprite data
			;
			ldx	#MaxSprites
			lda	#200+20
			sta	SpriteY

!InnerLoop:
			ldy	#6						; inner loop
			lda	#24
			sta	SpriteX
			lda	#0
			sta	SpriteX+1
!SetupSprites:
			lda	SpriteY
			sta	SpY,x

			lda	SpriteX
			sta	SpXLo,x
			lda	SpriteX+1
			sta	SpXHi,x

			lda	#33
			sta	SpShape,x

			lda	TempSpriteColours,x
			sta	SpCol,x

			; Fill in collision stuff
			lda	#12/2
			sta	CenterXoffset,x
			lda	#10
			sta	CenterYoffset,x
			lda	#11/2
			sta	RadX,x
			lda	#10
			sta	RadY,x

			lda	#Lo(Animation2)
			sta	AnimLo,x
			lda	#Hi(Animation2)
			sta	AnimHi,x
			lda	#-1				; frame = -1 for INIT
			sta	AnimFrame,x


			clc
			lda	SpriteX
			adc	#32
			sta	SpriteX
			lda	SpriteX+1
			adc	#0
			sta	SpriteX+1
			
			dex
			bmi	!FinishedSetup
			dey
			bpl	!SetupSprites


			sec
			lda	SpriteY
			sbc #21+13
			sta	SpriteY

			jmp	!InnerLoop
!FinishedSetup:
			lda	#100+24
			sta	SpXLo
			lda	#50+100-10
			sta	SpY
			lda	#0
			sta	SpXHi
			lda	#34
			sta	SpShape
			lda	#Lo(Animation1)
			sta	AnimLo
			lda	#Hi(Animation1)
			sta	AnimHi
			lda	#-1				; frame = -1 for INIT
			sta	AnimFrame



			;
			; Now create a random character graphic
			;
			lda	#$aa
			sta	CharacterSet
			lda	#$55
			sta	CharacterSet+1
			lda	#$ff
			sta	CharacterSet+2
			sta	CharacterSet+3
			sta	CharacterSet+4
			lda	#$55
			sta	CharacterSet+5
			lda	#0
			sta	CharacterSet+6
			sta	CharacterSet+7
			rts

;*****************************************************************************
;
; Move sprites around the screen
;
;*****************************************************************************
SpeedX		db	4		;
SpeedY		db	3		; Correct for aspect ratio
MovePlayer:
			lda	Joystick
			and	#JOY_UP
			beq	!TryDown
			sec
			lda	SpY
			sbc	SpeedY
			sta	SpY

!TryDown:	lda	Joystick
			and	#JOY_DOWN
			beq	!TryLeft
			clc
			lda	SpY
			adc	SpeedY
			sta	SpY

!TryLeft:	lda	Joystick
			and	#JOY_LEFT
			beq	!TryRight
			sec
			lda	SpXLo
			sbc	SpeedX
			sta	SpXLo
			lda	SpXHi
			sbc	#0
			sta	SpXHi

!TryRight:	lda	Joystick
			and	#JOY_RIGHT
			beq	!NoMovement
			clc
			lda	SpXLo
			adc	SpeedY
			sta	SpXLo
			lda	SpXHi
			adc	#0
			sta	SpXHi
!NoMovement
			inc	SpCol
			rts


;*****************************************************************************
;
; Move sprites around the screen
;
;*****************************************************************************

; This will cause them all to glitch badly, but should
; not cause BAD raster IRQ's
BounceOnY	equ	1				; Set to NON-ZERO to enable "Y" bounce too.



MoveSprites:
			ldx	#MaxSprites-1
ProcessAll:
			clc
			lda	SpXLo,x
			adc	SpriteSpeeds,x
			sta	SpXLo,x
			lda	SpXHi,x
			adc	SpriteSpeedsHi,x
			sta	SpXHi,x
	
	
			lda	SpXHi,x
			beq	!TestLow
			lda	SpXLo,x
			cmp	#Lo(320)
			bcc	!NoBounceX
!FlipXSpeed:
			lda	SpriteSpeeds,x
			eor	#$ff
			clc
			adc	#1					; Carry is SET
			sta	SpriteSpeeds,x
			lda	SpriteSpeedsHi,x	; Since the speed is always < 256 HI is never incremeted - just EOR'd
			eor	#$ff
			sta	SpriteSpeedsHi,x

			jmp	!NoBounceX
!TestLow	lda	SpXLo,x
			cmp	#24
			bcc	!FlipXSpeed
!NoBounceX
			;
			; Bounce on "Y" as well?
			; This will cause them all to glitch badly, but should
			; not cause BAD raster IRQ's
			;
			if BounceOnY != 0
DoY
			clc
			lda	SpY,x
			adc	SpriteSpeedsY,x
			sta	SpY,x
	
			lda	SpY,x	
			cmp	#50					; Bounce off the TOP of the screen.
			bcc	!FlipY
			cmp	#200+50-21			; Bounce off the bottom of the screen 
			bcc	!NoFlipY			; (200 pixels high, 50 pixel border off the top and -21 for the height of a sprite)
!FlipY
			lda	SpriteSpeedsY,x
			eor	#$ff
			sta	SpriteSpeedsY,x
			inc	SpriteSpeedsY,x
!NoFlipY
			endif
			dex	
			bne	ProcessAll

			rts

;*****************************************************************************
; Function:	Process Game Keys
;*****************************************************************************
ProcessKeys:
			lda	GameKeys
			bne	!KeysPressed
			rts
!KeysPressed:
			and	#KEY_F1
			beq	!F1_NotPressed
			lda	#1
			sta	$d021
	
!F1_NotPressed:
			lda	GameKeys
			and	#KEY_F3
			beq	!F3_NotPressed
			lda	#0
			sta	$d021
!F3_NotPressed:
			rts



;*****************************************************************************
; XPLY routine
;
; enter : xplyc = xp 1 ; xplyd = xp 2
;
;		  XPLYC can NOT be zero!!
;
; exit  : A = ans hi / xplyc = ans lo 
;
; used  : a
;
; Totals including RTS (not including jsr to here)
; 119 worst
; 101 best
;*****************************************************************************
XPLY:		
		LDA #$00		; 2
		dec	xplyc		; 5  pre-decrement to offset a SET carry when adding "xplyc"

		lsr xplyd		; 5
		BCC !skip1		; 2
		ADC xplyc		; 3	=10
!skip1
		ROR				; 2
		ROR xplyd		; 5
		BCC !skip2		; 3 = 10 (*9=90 best)
		ADC xplyc		;
!skip2
		ROR				; 2
		ROR xplyd		; 5
		BCC !skip3		; 2
		ADC xplyc		; 3 = 12 (*8=96 worst)
!skip3
		ROR
		ROR xplyd
		BCC !skip4
		ADC xplyc
!skip4
		ROR
		ROR xplyd
		BCC !skip5
		ADC xplyc
!skip5
		ROR
		ROR xplyd
		BCC !skip6
		ADC xplyc
!skip6
		ROR
		ROR xplyd
		BCC !skip7
		ADC xplyc
!skip7
		ROR
		ROR xplyd
		BCC !skip8
		ADC xplyc
!skip8
		ROR
		ROR xplyd
		BCC !skip9
		ADC xplyc
!skip9
	 	rts






;*****************************************************************************
; Simple Random Number
;
Random_Number:
	lda	#$11
	eor	Timera_2
	sta	Timerb_2
	
	lda	Timera_2+1
	eor	Timerb_2+1
	sta	Timerb_2+1
	clc
	adc	Timerb_2
	eor	Raster
	rts


;*****************************************************************************
;
; Function:		Simple player-2-sprite collision
; Description:	This does a simple center+radius collision check to the player
;				but can and should be heavily rewritten depending on the game.
;
;*****************************************************************************
PlayerYCenterOff	equ	10
PlayerXCenterOff	equ	12/2
PlayerWidth			equ	12/2
PlayerHeight		equ	10
PlayerCenterX		equ	Temp
PlayerCenterY		equ	Temp+1
CenterSubtraction	equ	Temp+2
DoCollision:
			clc
			lda	SpY					; Work out player center on Y
			adc	#PlayerYCenterOff	; by adding on a center offset
			sta	PlayerCenterY

			lda	SpXHi				; PlayerCenterX = (SpriteX/2)+(CenterOffset/2)
			lsr
			lda	SpXLo
			ror
			clc
			adc	#PlayerXCenterOff	; Center /2 because we've scaled back a bit
			sta	PlayerCenterX


			;
			; Now do actual player-2-sprite collision loop
			;
			ldx	#MaxSprites-1
!TestAll:
			lda	SpY,x				; Is this sprite active?
			beq	!SpriteNotActive

			;
			; First test collision on "X"
			;
			lda	SpXHi,x				; SpriteCenterX = (SpriteX/2)+(CenterXOffset/2)
			lsr
			lda	SpXLo,x
			ror
			clc
			adc	CenterXoffset,x		; SpriteCenterX-PlayerCenterX = CenterOffsetX			(already/2)
			sec
			sbc	PlayerCenterX
			bpl	!StoreX
			eor	#$ff				; Store ABS(SpriteCenterX-PlayerCenterX)
			adc	#1
!StoreX:	sta	CenterSubtraction

			clc						; Now get SpriteRadus+PlayerRadius = TotalRad
			lda	RadX,x
			adc	#PlayerWidth
			cmp	CenterSubtraction	; if(  TotalRad > CenterOffset ) then we've collided on Y
			bcc	!NoCollision


			;
			; If we get here, then we've collided on "X" so now try "Y"
			;
			lda	SpY,x
			clc
			adc	CenterYoffset,x		; SpriteCenter-PlayerCenter = CenterOffset
			sec						; (Player center could be offset by 1 to get rid of "sec")
			sbc	PlayerCenterY		; Subtract player center        
			bpl	!NotNeg1
			eor	#$ff				; abs(CenterOffset)			Carry is already clear
			adc	#1					
!NotNeg1:	sta	CenterSubtraction	; Store abs(CenterOffset)

			clc						; Now get SpriteRadus+PlayerRadius = TotalRad
			lda	RadY,x
			adc	#PlayerHeight
			cmp	CenterSubtraction	; if(  TotalRad > CenterOffset ) then we've collided on Y
			bcc	!NoCollision


			;
			; Do your collision code here!
			;
			if DEBUG = 1
;			lda	$D012				; wait till around the middle of the screen so we can see the RED BAR
;			CMP	#$80
;			bcc	DoCollision

			lda	#red
			sta	$d020
			ldy	#$10				; Collision currently shows a red bar.
!lp1		dey
			bne	!lp1
			lda	#black
			sta	$d020
			endif



!NoCollision:
!SpriteNotActive:
			dex
			bne	!TestAll 			; Dont do sprite "0" as thats the player!
			rts




;*****************************************************************************
;
; Function:		Simple player-2-sprite collision
; Description:	This does a simple center+radius collision check to the player
;				but can and should be heavily rewritten depending on the game.
;
;*****************************************************************************
Anim		equ	Temp

AnimateSprites:
			ldx	#MaxSprites
!AnimateAll	:
			lda	AnimLo,x
			sta	Anim
			lda	AnimHi,x
			sta	Anim+1
			ldy	AnimFrame,x
			bpl	!NoReset
!ResetAnim
			ldy	#0
			lda	(Anim),y
			sta	AnimDelay,x
			iny
			lda	(Anim),y
			sta	SpShape,x
			iny
			tya
			sta	AnimFrame,x
			bne	!NextSprite
!NoReset:
			dec	AnimDelay,x
			bne	!NextSprite

			lda	(Anim),y
			beq	!ResetAnim		; Depending on where your sprites are, this might have to change
			sta	SpShape,x
			iny
			tya
			sta	AnimFrame,x
			ldy	#0
			lda	(Anim),y
			sta	AnimDelay,x
!NextSprite
			dex
			bpl	!AnimateAll
			rts

			;
			;	Delay,   Frame,Frame,Frame,   Termination (0)
			;
Animation1:	db	250,	EmptySpriteShape+2, 0
Animation2:	db	2,		EmptySpriteShape+1, EmptySpriteShape+1,EmptySpriteShape+1,EmptySpriteShape+1,EmptySpriteShape+2,	0








