;******************************************************************************************************************
;
;  								             MMC64 Fileloader V1.0
;
;									                 By 
;
;									            Mike Dailly
;  								            (c) Copyright 2008
;                                          All Rights Reserved
;											
;                                             mike@dailly.org
;		
;								 Special Thanks to TNT for all his help
;	
; Terms of use:
;
;		This Source Code is given away free and you may do anything you like with it except
;		sell it. You should also seriously consider releasing the source to any game or application 
;		that uses any part of this program. Any new feature, addition or enhancement to any part of 
;		the framework must have its source code released for all to use. Such additions must also be 
;		be sent directly to me for possibly inclusion of the framework.
;
;		Credit must also be given to the original authors if any part of this source is used.
;
;		     Michael Dailly retains copyright of this program and its source+assets.
;
;
;	This program and its source were made to further the retro comunity, and breaking any of these conditions
;	renders all the years of work meaningless. So have fun with it, and then let others have fun with your work too!
;
;
; Directions for use
;--------------------
;	To initialise simply call Init_MMC64 like so
;
;				jsr	Init_MMC64			; Init the MMC64
;
;	To load a standard file, OR a .PRG file into a custom address
;
;				lda	#Lo(DestAddress)	; Initialise MMCDestAdd to the destination address
;				sta	MMCDestAdd
;				lda	#Hi(DestAddress)	
;				sta	MMCDestAdd+1	
;	
;				lda	#Lo(Filename)		; Load A & Y with the LO & HI byte of the filename
;				ldy	#Hi(Filename)		; The filename must be zero (0) terminated
;
;				jsr	LoadFile			; Load a NON .PRG file
;
;	The filename is in 8.3 format and must be UPPERCASE and null terminated
;
;				db	"TEST\TEST2\TEST3\SCREEN",0
;
;
;
;	To load a PRG file into its original place, simple load A&Y with the filename address
;	and then call LoadPRG
;
;				lda	#Lo(Filename)		; Load A & Y with the LO & HI byte of the filename
;				ldy	#Hi(Filename)		; The filename must be zero (0) terminated
;				jsr	LoadPRG				; Load a .PRG file into its original location
;
;
;******************************************************************************************************************

;********************************************
;
; SPI command sets
;
;********************************************
CMD0			equ		$40					; GO_IDLE_STATE
CMD1			equ		$40+1				; SEND_OP_COND 
CMD8			equ		$40+8				; SEND_IF_COND 
CMD9			equ		$40+9				; SEND_CSD 
CMD10			equ		$40+10				; SEND_CID 
CMD12			equ		$40+12				; STOP_TRANSMISSION 
CMD16			equ		$40+16				; SET_BLOCKLEN 
CMD17			equ		$40+17				; READ_SINGLE_BLOCK 
CMD18			equ		$40+18				; READ_MULTIPLE_BLOCK 
7CMD24			equ		$40+24				; WRITE_BLOCK 
CMD25			equ		$40+25				; WRITE_MULTIPLE_BLOCK
CMD41			equ		$40+41				; SEND_OP_COND (ACMD) 
CMD55			equ		$40+55				; APP_CMD 
CMD58			equ		$40+58				; READ_OCR 
CMD59			equ		$7b					; CRC_ON_OFF


;******************************************************************************************************************
; ZeroPage variables needed
;******************************************************************************************************************
;				Alloc	memptr,2					; MMC Work data used for sector loading
;				Alloc	NamePtr,2					; Pointer to current filename to load
;				Alloc	MMCDestAdd,2				; LOAD address
;				Alloc	MMCTemp,2					; General TEMP storage
;				Alloc	Sector,3
;				Alloc	CurrentCluster,2			; current cluster in search
;				Alloc	SectorCount,1				; Sectors left in this cluster



Start_MMC64_Code:
;******************************************************************************************************************
; DEBUG - Print a number in HEX
;
;	In:	A = number to print
;		Y = index into screen
;
;******************************************************************************************************************
	if 0
HexDigits		db	"0123456789",1,2,3,4,5,6
PrintHEX:		sty	HexOff1+1
				iny
				sty	HexOff2+1
				pha
				and	#$f
				tay
				lda	HexDigits,y
HexOff2			sta	$0701
				pla
				lsr
				lsr
				lsr
				lsr
				tay
				lda	HexDigits,y
HexOff1			sta	$0700
				ldy	HexOff1+1
				rts

;******************************************************************************************************************
; Print the current "Sector" number
;******************************************************************************************************************
PrintSector:
				tya
				clc
				adc	#4
				tay
				lda	Sector
				jsr	PrintHex
				lda	Sector+1
				dey
				dey
				jsr	PrintHex
				lda	Sector+2
				dey
				dey
				jmp	PrintHex

;******************************************************************************************************************
; Dump the current sector to the screen (then hang)
;******************************************************************************************************************
DumpSector:
!loop2			ldx	#0
!loop			lda	Buffer,x
				sta	$0400,x
				lda	Buffer+$100,x
				sta	$0500,x
				dex
				bne	!loop
				beq	!loop2
!infloop		jmp	!infloop


	endif		; End debug function

;;******************************************************************************************************************
;; 
;; Init the MMC64
;;
;;******************************************************************************************************************
;MMC64_Hardware_Init:
;
;				lda MMC64_Control		; Get contents of MMC64 control register
;				and #%10111011			; Set 250khz & write trigger mode
;				ora #%00000010			; Disable card select
;				sta MMC64_Control		; Update control register
;
;				ldx #$0a				; Initialise counter (10*8 pulses) 
;				ldy #$ff				; Initialise value to be written (bits must be all high)
;!mmcinitloop		
;				sty MMC64_SPI			; send 8 pulses to the card
;!busywait	
;				lda MMC64_Status		; We catch the status register
;				and #$01				; All bits shiftet out?
;				bne !busywait			; Nope, so wait 
;				dex						; Decrement counter
;				bne !mmcinitloop		; Until 80 pulses sent
;
;				lda MMC64_Control		; Pull card chip select line down for SPI communication
;				and #%11111101
;				sta MMC64_Control
;				rts
;
;;******************************************************************************************************************
;; 
;; Reset the MMC64 card	-	Execute a CMD0 command
;;
;;******************************************************************************************************************
;MMC64Reset
;				ldy	#$00				; we send 8 command bytes to the card
;
;!mmc64resetloop
;				lda	ResetCMD,y			; grab command byte
;				sta MMC64_SPI			; and fire it to the card
;!busy			lda MMC64_Status		; we check the busy bit
;				and #$01				; to ensure that the transfer is safe
;				bne !busy
;
;				iny						; decrease command counter
;				cpy	#8					  
;				bne !mmc64resetloop		; until the entire command has been sent
;
;				lda MMC64_SPI			; now we check the card response
;				and #$01				; did it accept the command ?
;				bne !mmc_responded		; ok, everything is fine !
;
;				lda MMC64_Status		; is there a card at all ?
;				and #$08		
;				beq MMC64Reset			; yup, so we just resend the command
;				sec
;				rts
;!mmc_responded
;				clc
;				rts
;	
;
;ResetCMD:		db	$ff				; SYNC
;				db	$40				; CMD0
;				db	0,0,0,0			; Don't care
;				db	$95				; CRC
;				db	$ff				; EndSync
;				;$ff,$ff,$95,$00,$00,$00,$00,$40,$ff	;CMD0
;
;
;
;;******************************************************************************************************************
;; 
;; Check if card has left initialization
;;
;;******************************************************************************************************************
;MMC64WaitInit
;				ldy #$00				; we send 8 command bytes to the cardEnableFastMode
;!initwaitloop
;				lda CMD1Data,y			; grab command byte
;				sta MMC64_SPI			; and fire it to the card
;!initwaitbusy
;				lda MMC64_Status		; we check the busy bit
;				and #$01				; to ensure that the transfer is safe
;				bne !initwaitbusy
;				iny						; decrease command counter	
;				cpy	#8
;				bne !initwaitloop		; until entire command has been sent
;
;				lda MMC64_SPI			; has the card left its init state?
;				and #$01
;				bne !mmcresponded			; no, so we resend the command to check it again
;
;				lda	MMC64_Status
;				and	#8
;				beq	MMC64WaitInit	
;!mmcresponded
;				rts
;	
;CMD1Data:		db	$ff				; SYNC
;				db	$41				; CMD1 command
;				db	0,0,0,0			; dont care
;				db	$ff				; CRC - dont care
;				db	$ff				; End byte
;
;; $ff,$ff,$ff,$00,$00,$00,$00,$41,$ff	;CMD1
;		



; ***********************************************************
; Name:		VerifyFATSector
; Function:	Check to see if the currently loaded sector is the
;			FAT boot record
;
; In:		Sector loaded into Buffer
; Out:		CLC = TRUE
;			SEC = FALSE
;
; ***********************************************************
VerifyFATSector:
				ldx	#0
FATBootCheck
				ldy	SectorVerify,x
				cpy	#0
				beq	!EndOfList
				lda	SectorVerify+1,x
				cmp	Buffer,y
				bne	!FatError
				inx
				inx
				bne	FATBootCheck
!EndOfList:
				clc
				rts
!FatError:
				sec
FoundFat_zero:				; Label needed for function below!!
				rts

				; Based on values provided by TNT
				; Value1 = FAT index
				; Value2 = Actual vaule
SectorVerify:
				db	$11,$00,	$12,$02					; Max number or ROOT directory entrys = $0200
				db	$0b,$00,	$0c,$02					; 512 bytes per sector
				db	$36,$46,	$37,$41,	$38,$54		; "FAT"
				db	$39,$31,	$3a,$36					; "16"
				db	$17,$00								; Sectors per FAT (top byte must be 0)
				db	$23,$00								; Sectors in Partition (Top byte of DWORD must be 0)
				db	$26,$29								; BootSIG = $29
				db	$15,$f8								; Media = HardDisk
				db	0





;******************************************************************************************************************
; Read the partition table and get the 1st partitions BOOT sector
;******************************************************************************************************************
ReadPartitionTable:
				lda	#0					; Read boot partition
				sta	Sector
				sta	Sector+1
				sta	Sector+2

				jsr	SetUpBufferAddress
				jsr	MMC64BlockRead


				jsr	VerifyFATSector
				bcc	FoundFat_zero

				lda	Buffer+$1be+8		; Start sector for this partition
				sta	Sector
				lda	Buffer+$1be+9
				sta	Sector+1
				lda	Buffer+$1be+10
				sta	Sector+2
LoadSector:
				jsr	SetUpBufferAddress
				; Fall through to block read!
				;jmp	MMC64BlockRead


;******************************************************************************************************************
;
; Block read command - set up a 512 byte transfer from
; $00_00_01_00 to $00_00_02_ff of the SD Card into C64 memory
;
; byte 1: 	$FF = sync header
; byte 2: 	$51 = command opcode	(CMD17 = $11 + $40)
; byte 3-6:	32bit memory address to read [31..0] (hi to low)
; byte 7:	CRC checksum (disabled by default, just write $FF)
; byte 8:	$FF = end sync header
;
;******************************************************************************************************************
MMC64BlockRead
				; Sector number *2 for 512 byte secotors
				lda	Sector				; LH
				asl
				pha
				lda	Sector+1			; HL
				rol
				pha
				lda	Sector+2			; HH
				rol


				ldy	#$ff
				sty MMC64_SPI			; SYNC
				ldx	#CMD17
				stx MMC64_SPI			; Command

				sta MMC64_SPI			; HIGH HIGH
				pla
				sta MMC64_SPI			; HIGH LOW
				pla
				sta MMC64_SPI			; Sector LOW HIGH
				lda	#0
				sta MMC64_SPI			; Sector LOW LOW (always 0)

				sty MMC64_SPI			; CRC (not in use)
				sty MMC64_SPI			; END SYNC header

!WaitReady:
				sty MMC64_SPI			; write all high bits
				lda MMC64_SPI			; to give the possibility to respond
				cmp	#$fe				; has it started?
				bne	!WaitReady 			; nop, so we continue waiting

				;
				; Now load the actual sector into memory (usually the sector buffer)
				;

				lda MMC64_Control		; set MMC64 into read trigger mode
 				ora #%01000000			; which means every read triggers a SPI transfer
 				sta MMC64_Control

				lda MMC64_SPI			; we have to start with one dummy read here

				ldx #$02				; set up counters
				ldy #$00
!sectorcopyloop
				lda MMC64_SPI			; get data byte from card
				sta (memptr),y			; store it into memory ( memptr has to be initialized)
				iny						
				lda MMC64_SPI			
				sta (memptr),y			
				iny
				lda MMC64_SPI			; Cut down on BNE instructions....
				sta (memptr),y			; Could save 1/10 of a second over large files.
				iny						
				lda MMC64_SPI			
				sta (memptr),y			
				iny						; 256 bytes copied yet?
				bne !sectorcopyloop		; nope, so go on!

				inc memptr+1			; increase memory pointer for next 256 bytes
				dex						; have we copied 512 bytes ?
				bne !sectorcopyloop

				lda MMC64_SPI			; we have to end the data transfer with one dummy read
	
				lda MMC64_Control		; now we put the hardware back into write trigger mode again
				and #%10111111	
				sta MMC64_Control
				rts

; ***********************************************************
; Get all the info we need from the boot sector
; (Boot Sector currently in Buffer)
; ***********************************************************
ProcessBootSector:
				lda	Buffer+13
				sta	SectorsPerCluster

				; SHOULDN'T need this as its almost always 512
				;lda	Buffer+17
				;sta	RootDirEntrys
				;lda	Buffer+18
				;sta	RootDirEntrys+1

				ldx	Buffer+16				; Get the total number of FATs

				; Not get the base sector of the RootDir
				; RootDir = BootSector+Reserver+(NumberOfFAT * SectorsPerFat)

				clc
				;lda	BootSector
				lda	Sector
				adc	ReservedSectors
				sta	DirRoot
				sta	FatRoot
				lda	Sector+1
				adc	ReservedSectors+1
				sta	DirRoot+1
				sta	FatRoot+1
				lda	Sector+2
				sta	DirRoot+2
				sta	FatRoot+2


				; Now total up the total space taken up by the FAT(s)
!AddAll			clc
				lda	DirRoot
				adc	SectorsPerFAT
				sta	DirRoot
				lda	DirRoot+1
				adc	SectorsPerFAT+1
				sta	DirRoot+1
				bcc	!SkipInc
				inc	DirRoot+2
!SkipInc:
				dex
				bne	!AddAll

				clc
				lda	DirRoot
				adc	#(512*32)/512			; (RootDirectoryEntrys*32)/BytesPerSector
				sta	DataRoot
				lda	DirRoot+1
				adc	#0
				sta	DataRoot+1
				lda	DirRoot+2
				adc	#0
				sta	DataRoot+2
				rts



;******************************************************************************************************************
;
; Name:		Init_MMC64
; Function:	Inialise and Enable the MMC64
;
; In:		None
; Out:		Carry CLEAR for okay
;			Carry SET for error
;
;******************************************************************************************************************
Init_MMC64:
				; Check to see if the MMC64 is enabled, and if not enable it!
				; Blah blah...dont work yet....

				; Enable FAST mode (8Mhz)
				lda MMC64_Control
				ora #%00000100
				sta MMC64_Control
			
				jsr	ReadPartitionTable
				jsr	VerifyFATSector
				bcs	!FatError

!AlreadyFoundBootSector:
				jsr	ProcessBootSector

				lda	#0
				sta	GameCluster				; ROOT dir is at cluster 0
				sta	GameCluster+1

				clc
				rts
!FatError:		sec			; fatel error
				rts

;******************************************************************************************************************
;
; 	Name:		ExtractNameSegment
;	Funcxtion:	Given a full path+filename, extract the next part of the name
;				to look for in a directory.
;
;	OnExit:		X=Length of name
;				SEC = NOT found
;				CLC = Found
;******************************************************************************************************************
ExtractNameSegment:
				ldy	NameIndex
				ldx	#0
!CopyAll		lda	(NamePtr),y
				beq	!FoundEndOfFilename
				cmp	#"\"					; End of a directory entry?
				beq	!FoundNextSearch
				cmp	#"."
				bne	!CarryOn

!PadWithSpace	cpx	#8						; 8.3 filenames are padded with space.
				bcs	!ignore					; so "TEST.TXT" becomes "TEST    TXT"
				lda	#$20
				sta	FindName,x
				inx
				bne	!PadWithSpace
!CarryOn:
				sta	FindName,x
				inx
!Ignore			iny
				bne	!CopyAll				; Dont make too long!!
!FoundNextSearch
!TerminateName
				cpx	#11						; Pad name out with spaces so it matches properly
				beq	!TerminateName2
				lda	#$20
				sta	FindName,x
				inx
				bne	!TerminateName
!TerminateName2:
				lda	#0
				sta	FindName,x				; Terminate string
				iny							; get past "\"
				sty	NameIndex				; Current index into name

				ldy	#2
!CheckAll		lda	FindName+8,y
				cmp	PRGText,y
				bne	!NotPRG
				dey
				bpl	!CheckAll

				sty	IsPRG					; if this is a ".PRG" flag it as such so it loads in the right place

!NotPRG			sec							; Flag MORE to search
				rts

!FoundEndOfFilename:
				cpx	#0
				beq	!EndOfFileName

				jsr	!TerminateName			; make sure we terminate the name properly...
				dec	NameIndex				; ALWAYS point to the 0
!EndOfFileName:
				clc							; flag END of search after this one
				rts


;******************************************************************************************************************
;
; 	Name:		SetUpBufferAddress
;	Funcxtion:	Setup the base address of the sector buffer
;
;******************************************************************************************************************
SetUpBufferAddress:
				lda	#Lo(Buffer)
				sta	memptr
				lda	#Hi(Buffer)
				sta	memptr+1
				rts


;******************************************************************************************************************
;
; 	Name:		SearchDirectory
;	Funcxtion:	Search the loaded sector buffer for the entry
;				that matches the current "FindName"
;
;******************************************************************************************************************
DirEntriesLeft	equ	MMCTemp
EntryBase		equ	MMCTemp+1


SearchDirectory:
				jsr	SetUpBufferAddress

				lda	#32
				sta	DirEntriesLeft

				ldy	#0
!SearchAllEntrys:
				ldx	#0
				sty	EntryBase
!SearchLoop
				lda	FindName,x
				beq	!FoundEntry
				cmp	(memptr),y
				bne	!NotThisEntry
				iny
				inx
				;cpx	#12					; MAX end of loop (should never need this...)
				bne	!SearchLoop
!NotThisEntry:	
				clc							; Carry is SET
				lda	EntryBase
				adc	#32						; -1 for SEC
				tay
				bne	!NotOver255
				inc	memptr+1
!NotOver255:	dec	DirEntriesLeft
				bne	!SearchAllEntrys
				sec
				rts

				; If we found the File/Directory, then get its starting
				; cluster and filesize
!FoundEntry:	clc		
				lda	EntryBase
				adc	#26+5-2
				tay
				ldx	#5-2					; Dont copy 2 HIGH bytes of file size... dont need them
!CopyAll		lda	(memptr),y
				sta	FileCluster,x
				dey
				dex
				bpl	!CopyAll
				clc
				rts

;******************************************************************************************************************
; Given the CLUSTER number, calculate the SECTOR number
; 
; Enter:	A,Y = Cluster Lo,Hi
;
;******************************************************************************************************************
GetRootDir
				ldy	#3
!copy			lda	DirRoot-1,y				; For now, the game root is the RootDir
				sta	Sector-1,y
				dey
				bne	!copy

				lda	#$20					; The  ROOT director is 32 sequencial sectors, NOT "SectorsPerCluster"
				sta	SectorCount
				rts

CalcSector:
				sta	Sector					; Store CLUSTER
				sty	Sector+1				; and get ready to SHIFT
				ora	Sector+1
				beq	GetRootDir
				lda	#0
				sta	Sector+2

				sec							; Cluster - 2 (there IS no cluster 0 or 1)
				lda	Sector
				sbc	#2
				sta	Sector
				bcs	!SkipDec
				dec	Sector+1
!SkipDec
				lda	SectorsPerCluster
!Calc			lsr
				bcs	!DoneCalc
				asl	Sector
				rol	Sector+1
				rol	Sector+2
				jmp	!Calc
!DoneCalc
				clc
				lda	Sector
				adc	DataRoot
				sta	Sector	
				lda	Sector+1
				adc	DataRoot+1
				sta	Sector+1
				lda	Sector+2
				adc	DataRoot+2
				sta	Sector+2

				lda	SectorsPerCluster		; Reload the number of sectors per cluster
				sta	SectorCount
				rts

;******************************************************************************************************************
; Get the next cluster in the chain. Use CurrentCluster as the current point
;******************************************************************************************************************
GetNextCluster:
				; Now divide cluster by 256 to get the FAT sector
				; (or just take the HI byte instead....)

				clc
				lda	FatRoot
				adc	CurrentCluster+1
				sta	Sector
				lda	FatRoot+1
				adc	#0
				sta	Sector+1
				lda	FatRoot+2
				adc	#0
				sta	Sector+2

				jsr	LoadSector		; Load up this FAT sector
				jsr	SetUpBufferAddress

				lda	CurrentCluster	; *2 for WORD access
				asl
				tay
				lda	#Hi(Buffer)
				adc	#0				; Qdd carry into HIGH byte of buffer address
				sta	memptr+1
	
				lda	(memptr),y
				sta	CurrentCluster
				iny
				lda	(memptr),y
				sta	CurrentCluster+1
EndOfSearch:							; Label from function below 
				clc						; Got to the END of the search!  (CLC for function below)
				rts





;******************************************************************************************************************
; Find the start of the actual file
;******************************************************************************************************************
FindFile:
				jsr	ExtractNameSegment	; Get the next part of the name to find.
				bcs	!SearchAgain		; NOT the end of the filename
				cpx	#0
				beq	EndOfSearch

!SearchAgain
				jsr	LoadSector
				jsr	SearchDirectory		; Search the directory for the desired file
				bcc	!Found
			
				dec	SectorCount
				bne	!NextSector	

				; If we've run out of sectors in this cluster, we need to get the 
				; next batch from the FAT
				jsr	GetNextCluster
				lda	CurrentCluster
				ldy	CurrentCluster+1
				cmp	#$ff				; Did we FIND the file?
				bne	!ContinueSearch
				cpy	#$ff
				bne	!ContinueSearch
				sec						; guess not
				rts

!NextSector:
				; If not found move onto NEXT sector
				inc	Sector				; Carry is SET
				bne	!SkipIncs
				inc	Sector+1
				bne	!SkipIncs
				inc	Sector+2
!SkipIncs:		jmp	!SearchAgain


				; If found then move to NEXT part of the name and search for THAT!
!Found			lda	FileCluster
				ldy	FileCluster+1
				sta	CurrentCluster
				sty	CurrentCluster+1
!ContinueSearch:
				jsr	CalcSector
				jmp	FindFile
				




;******************************************************************************************************************
;	Name:		LoadFile
;	Function:	Loads a file into "MMCDestAdd".
;				If you are loading a PRG via this function, the 1st 2 bytes are ignored
;
;	In:			A,Y 		=	"Filename Address",0
;				MMCDestAdd	=	Load address
;
;******************************************************************************************************************
LoadFile:		ldx	#$ff
				db	$2c			; opcode for BIT $0101


;******************************************************************************************************************
;	Name:		LoadPRG
;	Function:	Loads a PRG file into its default location.
;				Trying to load a normal file (not a .PRG) using this function will result in  an error.
;
;	In:			A,Y 		=	"Filename Address",0
;
;******************************************************************************************************************
LoadPRG:		ldx	#$00
				stx	OverRideLoadAddress

				sta	NamePtr			; Store filename
				sty	NamePtr+1
				lda	#0
				sta	NameIndex
				sta	IsPRG

				lda	GameCluster
				sta	CurrentCluster
				ldy	GameCluster+1
				sty	CurrentCluster+1
				jsr	CalcSector

				jsr	FindFile				; Given a filename, FIND it!
				bcc	!FileFound
				rts							; carry set on error

!FileFound
!Found			
				lda	FileCluster
				ldy	FileCluster+1
				sta	CurrentCluster
				sty	CurrentCluster+1
ContinueSearch:
				jsr	CalcSector
				;
				; Now we've FOUND the file, we can actually load it
				;
				; FileAttributes	holds the file's attributes
				; FileCluster 		holds the STARTing cluster of the file data
				; FileSize 			holds the filesize we need to load.
				;
CopyWholeFile
				; Is this the 1st sector of a PRG file?
				lda	IsPrg					; If this is a PRG file then we need to load the 1st sector into 
				bne	!LoadIntoSectorBuffer	; the sector buffer so we can get the proper load address first!

				; Now load 512 bytes
				lda	FileSize+1				; if >512 bytes, load sector DIRECTLY into 
				cmp	#2						; the destination memory location
				bcc	!LessThan512Bytes

				lda	MMCDestAdd
				sta	memptr
				lda	MMCDestAdd+1
				sta	memptr+1
				jsr	MMC64BlockRead			; Read sector DIRECTLY into memory

				inc	MMCDestAdd+1
				inc	MMCDestAdd+1
				dec	FileSize+1
				dec	FileSize+1
				jmp	!Next512ByteBlock
!LoadIntoSectorBuffer
!LessThan512Bytes:
				jsr	LoadSector
				dec	memptr+1				; back to base of buffer
				dec	memptr+1				; back to base of buffer


				; Load PRG files...
				lda	IsPrg					; If this is a PRG file then we need to load the 1st sector into 
				beq	!DontSkip1st2bytes		; the sector buffer so we can get the proper load address first!
				lda	OverRideLoadAddress		; NO! Dont use the 1st 2 bytes as load address
				bne	!DontUSELoadAddress		; just skip them
				lda	Buffer					; get LOAD address of PRG
				sta	MMCDestAdd
				lda	Buffer+1
				sta	MMCDestAdd+1
!DontUSELoadAddress:
				sec
				lda	MMCDestAdd				; Offset DEST address by -2 to compensate for "load address"
				sbc	#2
				sta	MMCDestAdd
				bcs	!SkipDec
				dec	MMCDestAdd+1
!SkipDec		lda	#2						; set "Y" offset
				sta	IsPrg

!DontSkip1st2bytes:

				ldx	#2
!CopyMore
				lda	FileSize+1				; upto 64k in a load...
				;ora	FileSize+2			; allow for SCPU			(C64 version cant be more than 64K)
				bne	!MoreThat256Bytes		; Do Small Load
				lda	FileSize
				beq	AllDone

				; Copy left over bits
				ldy	IsPrg					; If this is a PRG file then we need to load the 1st sector into 
!CopyLeftOvers	lda	(memptr),y
				sta	(MMCDestAdd),y
				iny
				cpy	FileSize				; Must count UP due to IsPrg being "0" or "2"
				bne	!CopyLeftOvers
				lda	(memptr),y				; do "0" as well - cant do BPL as FileSize could be >128
				sta	(MMCDestAdd),y
				jmp	AllDone


!MoreThat256Bytes
				ldy	IsPrg					; If this is a PRG file then we need to load the 1st sector into 
!CopyAll		lda	(memptr),y
			   	sta	(MMCDestAdd),y
				iny
				bne	!CopyAll
				sty	IsPrg

				inc	MMCDestAdd+1		; Next page
				inc	memptr+1			; inc buffer address
				dec	FileSize+1			; C64 version is ALWAYS <64K

				dex
				bne	!CopyMore
!Next512ByteBlock:
				lda	#0
				sta	IsPrg

				; Next sector
				inc	Sector
				bne	!SkipInc
				inc	Sector+1
				bne	!SkipInc
				inc	Sector+2
!SkipInc		
				dec	SectorCount
				jne	CopyWholeFile
				
				jsr	GetNextCluster
				lda	CurrentCluster
				ldy	CurrentCluster+1
				jmp	ContinueSearch

AllDOne:		clc
				rts


;******************************************************************************************************************
;
;  MMC loader DATA area
;
;******************************************************************************************************************
MMC64DataStart:
MAX_FILENAME		equ	12				; Max length of a filename we can load
BytesPerSector		equ	512

PRGText				db	"PRG"
IsPRG				db	0				; !=0 if file is a .PRG file (so autoloads to its own address)
OverRideLoadAddress	db	0				; if !=0 then we want to override the load address of the PRG file
NameIndex			db	0				; Distance into name during file/dir search
FindName			ds	MAX_FILENAME+1

FileAttributes		db	0				; File attributes
FileCluster			db	0,0				; File search cluster
FileSize			db	0,0 ;,0,0		; File Size in BYTES (for a stock C64, its <64k!)

SectorsPerCluster	db	0				; Number of sectors per cluster
FatRoot				db	0,0,0			; Starting sector of the 1st FAT
DirRoot				db	0,0,0			; the root directory sector
DataRoot			db	0,0,0			; The base of cluster 2 - Start of all data
GameCluster			db	0,0				; the games root directory cluster
Buffer				ds	512				; Sector Buffer

ReservedSectors		equ	Buffer+14
SectorsPerFAT		equ	Buffer+22








; Size includes 512byte buffer
End_MMC64_Code
				Message "MMC Code = ", (MMC64DataStart-Start_MMC64_Code)
				Message "MMC Data = ", (End_MMC64_Code-MMC64DataStart)



